@@ -50,18 +50,6 @@ namespace Lum {
     */
     class LUMAPI Path
     {
-    public:
-      class LUMAPI PathDescription
-      {
-      public:
-        std::wstring description;
-        std::wstring path;
-
-      public:
-        PathDescription(const std::wstring& description, const std::wstring& path);
-  virtual ~PathDescription();
-      };
-
     private:
       std::vector<std::wstring> parts;
       std::wstring              root;
@@ -102,18 +90,8 @@ namespace Lum {
       static Status GetAbsoluteFilename(const std::wstring& fileName, std::wstring& absoluteFilename);
 
       static std::wstring GetCWD();
-      static std::wstring GetDirFromEnvironment(const std::string& environment);
-      static std::wstring GetHome();
-      static std::wstring GetTemporary();
-      static std::wstring GetConfigDir();
       static std::wstring GetApplicationConfigFile();
       static std::wstring GetApplicationConfigPath();
-      static void GetStdPathes(std::vector<PathDescription>& list);
-
-      static std::wstring GetLibraryDataDir();
-
-      static void SetApplicationDataDir(const std::wstring& directory);
-      static std::wstring GetApplicationDataDir();
 
       static Status CreateDir(const std::wstring& nativePath);
       static Status RemoveDir(const std::wstring& nativePath);
