@@ -21,7 +21,7 @@
 */
 
 #include <Lum/Model/Action.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 
 #include <Lum/OS/Thread.h>
 
