@@ -22,7 +22,7 @@
 
 #include <Lum/Object.h>
 
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 
 namespace Lum {
 
@@ -45,7 +45,8 @@ namespace Lum {
     void CalcSize();
     void Layout();
 
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
 
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
 
