@@ -50,17 +50,25 @@ namespace Lum {
     private:
       std::list<Loader*>    loader;
       std::list<Base::Path> applicationSearchPaths;
+      std::wstring          lightThemePostfix;
+      std::wstring          darkThemePostfix;
       std::wstring          normalImagePostfix;
       std::wstring          normalSelectedImagePostfix;
       std::wstring          disabledImagePostfix;
       std::wstring          disabledSelectedImagePostfix;
       std::wstring          activatedImagePostfix;
 
+    private:
+      bool LoadApplication(const std::wstring& basename,
+                           ImageRef &image, bool showError) const;
+
     public:
       Manager();
       ~Manager();
 
       Manager& AddApplicationSearchPath(const Base::Path& path);
+      Manager& SetLightThemePostfix(const std::wstring& postfix);
+      Manager& SetDarkThemePostfix(const std::wstring& postfix);
       Manager& SetNormalImagePostfix(const std::wstring& postfix);
       Manager& SetNormalSelectedImagePostfix(const std::wstring& postfix);
       Manager& SetDisabledImagePostfix(const std::wstring& postfix);
