@@ -102,14 +102,11 @@ namespace Lum {
       size_t GetRows() const;
       size_t GetColumns() const;
 
-      std::wstring GetString(size_t column, size_t row) const;
-      Lum::Object* GetObject(size_t column, size_t row) const;
-
       void SetFilter(Filter* filter);
       void SetDirectory(const std::wstring& directory,
                         bool noHiddenFiles, bool onlyDirs);
 
-      void GetEntry(size_t row, Base::DirScanner::Entry& entry);
+      void GetEntry(size_t row, Base::DirScanner::Entry& entry) const;
     };
 
     typedef Base::Reference<Dir> DirRef;
