@@ -20,6 +20,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
+#include <limits>
 #include <string>
 
 #include <Lum/Model/Simple.h>
@@ -29,10 +30,42 @@ namespace Lum {
 
     class LUMAPI String : public Simple<std::wstring>
     {
-    public:
-      String() : Simple<std::wstring>() {}
-      String(const std::wstring& value) : Simple<std::wstring>(value) {}
+    private:
+      size_t maxLength;
 
+    public:
+      String()
+      : Simple<std::wstring>(),
+        maxLength(std::numeric_limits<size_t>::max())
+      {
+        // no code
+      }
+
+      String(size_t maxLength)
+      : Simple<std::wstring>(),
+        maxLength(maxLength)
+      {
+        // no code
+      }
+
+      String(const std::wstring& value)
+      : Simple<std::wstring>(value),
+        maxLength(std::numeric_limits<size_t>::max())
+      {
+        // no code
+      }
+
+      String(const std::wstring& value, size_t maxLength)
+      : Simple<std::wstring>(value),
+        maxLength(maxLength)
+      {
+        // no code
+      }
+
+      void SetMaxLength(size_t maxLength);
+      size_t GetMaxLength() const;
+
+      void Set(const std::wstring& value);
       void Erase(size_t start, size_t length);
       void Insert(size_t pos, const std::wstring& text);
       size_t Length() const;
