@@ -30,13 +30,15 @@ namespace Lum {
   {
   public:
     void CalcSize();
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
   };
 
   class LUMAPI VScale : public Scale
   {
     void CalcSize();
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
   };
 }
 
