@@ -22,7 +22,7 @@
 
 #include <Lum/Base/Size.h>
 
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 
 #include <Lum/OS/Image.h>
 
@@ -48,7 +48,7 @@ namespace Lum {
     size_t         knobHeight;
 
   protected:
-    virtual void DrawKnob() = 0;
+    virtual void DrawKnob(OS::DrawInfo* draw) = 0;
 
   public:
     Slider();
@@ -68,7 +68,7 @@ namespace Lum {
   {
   private:
     void HandleMouseMovement(int x, int y);
-    void DrawKnob();
+    void DrawKnob(OS::DrawInfo* draw);
 
   public:
     HSlider();
@@ -78,7 +78,8 @@ namespace Lum {
 
     void CalcSize();
     void Layout();
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
 
     bool HandleKeyEvent(const OS::KeyEvent& event);
     bool HandleMouseEvent(const OS::MouseEvent& event);
@@ -95,14 +96,15 @@ namespace Lum {
   {
   private:
     void HandleMouseMovement(int x, int y);
-    void DrawKnob();
+    void DrawKnob(OS::DrawInfo* draw);
 
   public:
     VSlider();
 
     void CalcSize();
     void Layout();
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
 
     bool HandleKeyEvent(const OS::KeyEvent& event);
     bool HandleMouseEvent(const OS::MouseEvent& event);
