@@ -23,16 +23,19 @@
 #include <vector>
 
 #include <Lum/Base/DateTime.h>
+#include <Lum/Base/Reference.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/Header.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 #include <Lum/Model/Selection.h>
 #include <Lum/Model/Table.h>
 
 #include <Lum/Object.h>
+#include <Lum/ValuePainter.h>
 
 namespace Lum {
+
   class LUMAPI TableView : public Scrollable
   {
   private:
@@ -57,16 +60,6 @@ namespace Lum {
     };
 
   public:
-    class LUMAPI FormatProvider
-    {
-    public:
-      virtual ~FormatProvider();
-
-      virtual Alignment GetAlignment(size_t column, size_t row) const;
-      virtual OS::Color GetTextColor(size_t column, size_t row) const;
-      virtual OS::Fill* GetBackground(size_t column, size_t row) const;
-    };
-
     class Controller : public ::Lum::Controller
     {
     public:
@@ -81,6 +74,9 @@ namespace Lum {
     Model::SelectionRef   selection;            //! The selection mode
     Model::ActionRef      doubleClickAction;    //! Action to be triggered on double click
     Model::ActionRef      mouseSelectionAction;
+    Model::ActionRef      defaultAction;        //! A non-destructive default action
+                                                //! triggered by selection or doubleclick
+                                                //! depending on the environment
 
     bool                  autoFitColumns;
     bool                  autoHSize;
@@ -89,20 +85,20 @@ namespace Lum {
     size_t                tableWidth;
     size_t                rowHeight;
 
+    size_t                refreshes;
     bool                  fullRefresh;
     size_t                top;
 
-    FormatProvider        *formatProvider;
+    TableCellPainterRef   painter;
     OS::FontRef           font;
 
   private:
-    void UpdateDimensions();
-
     bool GetCell(size_t mx, size_t my, size_t& x, size_t& y) const;
+    void ResyncWithModel();
     void RecalcTableWidth();
     void FitColumn(size_t column);
     void CalculateRowHeight();
-    void FitColumns();
+    void FitColumns(size_t targetWidth);
     bool GetLineBounds(size_t line, int& y, size_t& height) const;
 
     void FillWithBackground(OS::DrawInfo* draw, int xPos, int yPos, size_t width, size_t height);
@@ -131,25 +127,32 @@ namespace Lum {
     bool SetModel(Base::Model* model);
     void SetHeader(Model::Header *header);
     void SetSelection(Model::Selection* selection);
+
     void SetMouseSelectionAction(Model::Action* action);
     void SetDoubleClickAction(Model::Action* action);
-    void SetFormatProvider(FormatProvider* formatProvider);
+    void SetDefaultAction(Model::Action* action);
+
+    void SetPainter(TableCellPainter* painter);
+    TableCellPainter* GetPainter() const;
 
     // Getter
     Model::Table* GetModel() const;
     Model::Selection* GetSelection() const;
     Model::Action* GetDoubleClickAction() const;
     Model::Action* GetMouseSelectionAction() const;
+    Model::Action* GetDefaultAction() const;
 
     void MakeVisible(size_t x, size_t y);
 
     void CalcSize();
-
+    void GetDimension(size_t& width, size_t& height);
     void Layout();
 
-    OS::Color GetTextColor(OS::DrawInfo *draw) const;
+    OS::Color GetTextColor(OS::DrawInfo *draw,
+                           const void* child) const;
 
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
     void Hide();
     void DrawFocus();
     void HideFocus();
