@@ -36,18 +36,7 @@ namespace Lum {
     delete object;
   }
 
-  bool AspectRatio::VisitChildren(Visitor &visitor, bool onlyVisible)
-  {
-    if (object!=NULL) {
-      if (!visitor.Visit(object)) {
-        return false;
-      }
-    }
-
-    return true;
-  }
-
-  void AspectRatio::SetAspectRatio(size_t hAspect, size_t vAspect)
+  AspectRatio* AspectRatio::SetAspectRatio(size_t hAspect, size_t vAspect)
   {
     this->hAspect=hAspect;
     this->vAspect=vAspect;
@@ -55,9 +44,11 @@ namespace Lum {
     if (object!=NULL && visible) {
       Redraw();
     }
+
+    return this;
   }
 
-  void AspectRatio::SetAlignment(HAlignment halign, VAlignment valign)
+  AspectRatio* AspectRatio::SetAlignment(HAlignment halign, VAlignment valign)
   {
     bool redraw;
 
@@ -69,14 +60,11 @@ namespace Lum {
     if (redraw) {
       Redraw();
     }
+
+    return this;
   }
 
-  Object* AspectRatio::GetObject() const
-  {
-    return object;
-  }
-
-  void AspectRatio::SetObject(Object* object)
+  AspectRatio* AspectRatio::SetObject(Object* object)
   {
     assert(object!=NULL);
 
@@ -85,6 +73,24 @@ namespace Lum {
     this->object=object;
     this->object->SetParent(this);
     this->object->SetFlex(true,true);
+
+    return this;
+  }
+
+  Object* AspectRatio::GetObject() const
+  {
+    return object;
+  }
+
+  bool AspectRatio::VisitChildren(Visitor &visitor, bool onlyVisible)
+  {
+    if (object!=NULL) {
+      if (!visitor.Visit(object)) {
+        return false;
+      }
+    }
+
+    return true;
   }
 
   void AspectRatio::CalcSize()
@@ -152,5 +158,35 @@ namespace Lum {
 
     Group::Layout();
   }
+
+  AspectRatio* AspectRatio::Create(Object* object,
+                                   size_t hAspect, size_t vAspect,
+                                   bool horizontalFlex, bool verticalFlex)
+  {
+    AspectRatio* o;
+
+    o=new AspectRatio();
+    o->SetFlex(horizontalFlex,verticalFlex);
+    o->SetAspectRatio(hAspect,vAspect);
+    o->SetObject(object);
+
+    return o;
+  }
+
+  AspectRatio* AspectRatio::Create(Object* object,
+                                   size_t hAspect, size_t vAspect,
+                                   HAlignment halign, VAlignment valign,
+                                   bool horizontalFlex, bool verticalFlex)
+  {
+    AspectRatio* o;
+
+    o=new AspectRatio();
+    o->SetFlex(horizontalFlex,verticalFlex);
+    o->SetAspectRatio(hAspect,vAspect);
+    o->SetAlignment(halign,valign);
+    o->SetObject(object);
+
+    return o;
+  }
 }
 
