@@ -28,6 +28,8 @@
 
 #include <Lum/Config/Config.h>
 
+#include <Lum/Manager/FileSystem.h>
+
 #include <Lum/OS/Display.h>
 
 namespace Lum {
@@ -164,16 +166,26 @@ namespace Lum {
 #endif
     }
 
-    std::wstring L10N::GetLocalizedText(const std::wstring& key, const std::wstring& defaultValue)
+    std::wstring L10N::GetLocalizedText(const std::wstring& key,
+                                        const std::wstring& defaultValue)
     {
       if (!appCatalogSearched) {
-        appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(locale,Path::GetApplicationDataDir(),Catalog::typeApplication));
-        if (appCatalog.get()!=NULL) {
-          appCatalogFallback=(StripSubData(locale)==StripSubData(fallbackLocale));
-        }
-        else {
-          appCatalogFallback=true;
-          appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(fallbackLocale,Path::GetApplicationDataDir(),Catalog::typeApplication));
+        std::wstring appDataDir;
+
+        if (Manager::FileSystem::Instance()->GetEntry(Manager::FileSystem::appDataDir,
+                                                      appDataDir)) {
+          appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(locale,
+                                                                 appDataDir,
+                                                                 Catalog::typeApplication));
+          if (appCatalog.get()!=NULL) {
+            appCatalogFallback=(StripSubData(locale)==StripSubData(fallbackLocale));
+          }
+          else {
+            appCatalogFallback=true;
+            appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(fallbackLocale,
+                                                                   appDataDir,
+                                                                   Catalog::typeApplication));
+          }
         }
         appCatalogSearched=true;
       }
@@ -186,10 +198,19 @@ namespace Lum {
       }
     }
 
-    std::wstring L10N::GetLocalizedLibText(const std::wstring& key, const std::wstring& defaultValue)
+    std::wstring L10N::GetLocalizedLibText(const std::wstring& key,
+                                           const std::wstring& defaultValue)
     {
       if (!libCatalogSearched) {
-        libCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(locale,Path::GetLibraryDataDir(),Catalog::typeLibrary));
+        std::wstring libDataDir;
+
+        if (Manager::FileSystem::Instance()->GetEntry(Manager::FileSystem::libDataDir,
+                                                      libDataDir)) {
+          libCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(locale,
+                                                                 libDataDir,
+                                                                 Catalog::typeLibrary));
+        }
+
         libCatalogSearched=true;
       }
 
@@ -276,7 +297,7 @@ namespace Lum {
         paths.push_back(catalogPath);
       }
       else if (type==typeLibrary) {
-        tmp=Path::GetDirFromEnvironment("ILLUMINATION_ROOT");
+        tmp=Manager::FileSystem::Instance()->GetDirFromEnvironment("ILLUMINATION_ROOT");
         if (!tmp.empty()) {
           catalogPath.SetNativeDir(tmp);
           catalogPath.AppendDir(L"catalogs");
