@@ -259,9 +259,10 @@ namespace Lum {
     }
   }
 
-  void DatePicker::Draw(int x, int y, size_t w, size_t h)
+  void DatePicker::Draw(OS::DrawInfo* draw,
+                        int x, int y, size_t w, size_t h)
   {
-    Control::Draw(x,y,w,h);
+    Control::Draw(draw,x,y,w,h);
 
     if (!Intersect(x,y,w,h)) {
       return;
@@ -269,14 +270,12 @@ namespace Lum {
 
     /* --- */
 
-    OS::DrawInfo *draw=GetDrawInfo();
+    current->Draw(draw,x,y,w,h);
 
-    current->Draw(x,y,w,h);
-
-    prevYearButton->Draw(x,y,w,h);
-    prevMonthButton->Draw(x,y,w,h);
-    nextMonthButton->Draw(x,y,w,h);
-    nextYearButton->Draw(x,y,w,h);
+    prevYearButton->Draw(draw,x,y,w,h);
+    prevMonthButton->Draw(draw,x,y,w,h);
+    nextMonthButton->Draw(draw,x,y,w,h);
+    nextYearButton->Draw(draw,x,y,w,h);
 
     for (size_t Y=0; Y<7; Y++) {
       for (size_t X=0; X<7; X++) {
@@ -289,7 +288,7 @@ namespace Lum {
                                 texts[text]->GetOWidth(),texts[text]->GetOHeight());
             draw->selected=true;
           }
-          texts[text]->Draw(x,y,w,h);
+          texts[text]->Draw(draw,x,y,w,h);
 
           if (text==cS) {
             draw->selected=false;
