@@ -21,6 +21,8 @@
 
 #include <Lum/WindowGroup.h>
 
+#include <Lum/Manager/Behaviour.h>
+
 namespace Lum {
 
   Dialog::Dialog()
@@ -37,7 +39,7 @@ namespace Lum {
     Observe(GetWindow()->GetMenuAction());
 
     if (OS::display->GetTheme()->GetMenuType()==OS::Theme::menuTypeCustom) {
-      customMenuHandler=OS::display->GetBehaviour()->GetCustomMenuHandler(GetWindow());
+      customMenuHandler=Manager::Behaviour::Instance()->GetCustomMenuHandler(GetWindow());
     }
   }
 
@@ -149,6 +151,7 @@ namespace Lum {
     else {
       group->SetBackground(OS::display->GetFill(OS::Display::backgroundFillIndex));
     }
+    
     group->SetFlex(main->HorizontalFlex(),main->VerticalFlex());
 
     group->SetToolbar(toolbar);
