@@ -8,8 +8,11 @@
 
 #include <Lum/Base/L10N.h>
 #include <Lum/Base/Path.h>
+#include <Lum/Base/String.h>
 #include <Lum/Base/Util.h>
 
+#include <Lum/Manager/FileSystem.h>
+
 #include <Lum/Model/Selection.h>
 #include <Lum/Model/Table.h>
 
@@ -40,18 +43,20 @@ namespace Lum {
     typedef Model::StdTable<RootsEntry> RootsModel;
     typedef Base::Reference<RootsModel> RootsModelRef;
 
-    class RootsDataProvider : public RootsModel::DataProvider
+    class RootsModelPainter : public StringCellPainter
     {
     public:
-      std::wstring GetString(const RootsModel::Iterator& iter, size_t column) const
+      std::wstring GetCellData() const
       {
-        switch (column) {
+        const RootsEntry entry=dynamic_cast<const RootsModel*>(GetModel())->GetEntry(GetRow());
+
+        switch (GetColumn()) {
         case 1:
-          return iter->name;
+          return entry.name;
         case 2:
-          return iter->path;
+          return entry.path;
         default:
-          assert(false);
+          return L"";
         }
       }
     };
@@ -68,22 +73,25 @@ namespace Lum {
       Locations()
       : table(new Table()),
         selection(new Model::SingleLineSelection),
-        roots(new RootsModel(new RootsDataProvider()))
+        roots(new RootsModel())
       {
         Observe(table->GetTableView()->GetMouseSelectionAction());
       }
 
       void PreInit()
       {
-        std::vector<Base::Path::PathDescription> list;
+        std::list<Manager::FileSystem::Place> places;
 
-        Base::Path::GetStdPathes(list);
+        Manager::FileSystem::Instance()->GetPlaces(places);
 
         Model::HeaderRef headerModel=new Model::HeaderImpl();
         headerModel->AddColumn(L"",Lum::Base::Size::pixel,0);
 
-        for (size_t x=0; x<list.size(); x++) {
-          roots->Append(RootsEntry(list[x].description,list[x].path));
+        for (std::list<Manager::FileSystem::Place>::const_iterator place=places.begin();
+             place!=places.end();
+             ++place) {
+          roots->Append(RootsEntry(place->description,
+                                   place->path));
         }
 
         table->SetFlex(true,true);
@@ -93,9 +101,10 @@ namespace Lum {
         table->SetHeaderModel(headerModel);
         table->SetSelection(selection);
         table->SetModel(roots);
-        table->GetTableView()->SetAutoFitColumns(true);
-        table->GetTableView()->SetAutoHSize(true);
-        table->GetTableView()->SetAutoVSize(true);
+        table->SetPainter(new RootsModelPainter());
+        table->SetAutoFitColumns(true);
+        table->SetAutoHSize(true);
+        table->SetAutoVSize(true);
         RegisterCommitShortcut(table->GetTableView(),
                                table->GetTableView()->GetMouseSelectionAction());
 
@@ -161,20 +170,41 @@ namespace Lum {
       defaultFilter=0;
     }
 
-    class FileListFormatProvider : public TableView::FormatProvider
+    class DirModelPainter : public StringCellPainter
     {
     public:
-      TableView::Alignment GetAlignment(size_t column, size_t /*row*/) const
+      DirModelPainter()
       {
-        switch (column) {
+        SetAlignment(1,left);
+        SetAlignment(2,right);
+        SetAlignment(3,right);
+      }
+
+      std::wstring GetCellData() const
+      {
+        Base::DirScanner::Entry entry;
+
+        dynamic_cast<const Model::Dir*>(GetModel())->GetEntry(GetRow(),entry);
+
+        switch (GetColumn()) {
         case 1:
-          return TableView::left;
+          return entry.name;
         case 2:
-          return TableView::right;
+          if (entry.isDir) {
+            return _l(L"MODEL_DIR_DIR",L"(dir)");
+          }
+          else {
+            return Base::ByteSizeToWString(entry.size);
+          }
         case 3:
-          return TableView::right;
+          if (entry.timeModification.IsToday()) {
+            return entry.timeModification.GetLocalLocaleTime();
+          }
+          else {
+            return entry.timeModification.GetLocalLocaleDate();
+          }
         default:
-          return TableView::left;
+          return L"";
         }
       }
     };
@@ -214,7 +244,6 @@ namespace Lum {
       Observe(GetOpenedAction());
       Observe(GetClosedAction());
       Observe(path);
-      Observe(full);
       Observe(filter);
       Observe(showHidden);
       Observe(okAction);
@@ -268,7 +297,7 @@ namespace Lum {
 
       header=new Model::HeaderImpl();
       header->AddColumn(_l(L"MODEL_DIR_FILENAME",L"Filename"),Base::Size::stdCharWidth,15,true);
-      header->AddColumn(_l(L"MODEL_DIR_SIZE",L"Size"),Base::Size::stdCharWidth,9);
+      header->AddColumn(_l(L"MODEL_DIR_SIZE",L"Size"),Base::Size::stdCharWidth,std::max(8u,_l(L"MODEL_DIR_DIR",L"(dir)").length()));
       header->AddColumn(_l(L"MODEL_DIR_DATE",L"Date"),Base::Size::stdCharWidth,10);
 
       table=new Table();
@@ -279,9 +308,9 @@ namespace Lum {
       table->GetTableView()->SetAutoHSize(true);
       table->SetSelection(selection);
       table->SetDoubleClickAction(doubleClickAction);
-      table->SetFormatProvider(new FileListFormatProvider());
       table->SetModel(dirList);
       table->SetHeaderModel(header);
+      table->SetPainter(new DirModelPainter());
 
       popup=new Menu();
       popup->SetParent(GetWindow());
@@ -623,7 +652,7 @@ namespace Lum {
                fullAction->IsFinished()) {
         HandleChange();
       }
-      else if (model==selection) {
+      else if (selection->HasSelectionChanged(model,msg)) {
         OnSelectionChange();
       }
       else if (model==doubleClickAction &&
@@ -674,7 +703,7 @@ namespace Lum {
                        std::wstring& file,
                        Options* options)
     {
-      File *dialog;
+      File dialog;
       bool result=false;
 
       if (options==NULL) {
@@ -683,24 +712,21 @@ namespace Lum {
       options->SetMode(Options::modeFile);
       options->SetExistingOnly(true);
 
-      dialog=new File();
-      dialog->SetParent(parent);
-      dialog->SetTitle(title);
-      dialog->SetOptions(options);
-      dialog->SetPath(file);
+      dialog.SetParent(parent);
+      dialog.SetTitle(title);
+      dialog.SetOptions(options);
+      dialog.SetPath(file);
 
-      if (dialog->Open()) {
-        dialog->EventLoop();
-        dialog->Close();
+      if (dialog.Open()) {
+        dialog.EventLoop();
+        dialog.Close();
 
-        if (dialog->GetResult()) {
-          file=dialog->GetPath();
+        if (dialog.GetResult()) {
+          file=dialog.GetPath();
           result=true;
         }
       }
 
-      delete dialog;
-
       return result;
     }
 
@@ -745,7 +771,7 @@ namespace Lum {
                       std::wstring& dir,
                       Options* options)
     {
-      File *dialog;
+      File dialog;
       bool result=false;
 
       if (options==NULL) {
@@ -755,24 +781,21 @@ namespace Lum {
       options->SetMode(Options::modeDirectory);
       options->SetExistingOnly(true);
 
-      dialog=new File();
-      dialog->SetParent(parent);
-      dialog->SetTitle(title);
-      dialog->SetOptions(options);
-      dialog->SetPath(dir);
+      dialog.SetParent(parent);
+      dialog.SetTitle(title);
+      dialog.SetOptions(options);
+      dialog.SetPath(dir);
 
-      if (dialog->Open()) {
-        dialog->EventLoop();
-        dialog->Close();
+      if (dialog.Open()) {
+        dialog.EventLoop();
+        dialog.Close();
 
-        if (dialog->GetResult()) {
-          dir=dialog->GetPath();
+        if (dialog.GetResult()) {
+          dir=dialog.GetPath();
           result=true;
         }
       }
 
-      delete dialog;
-
       return result;
     }
   }
