@@ -23,9 +23,28 @@
 
 namespace Lum {
 
-  LED::LED()
-   : ImageState(OS::Theme::objLED,false)
+  LED::LED(Base::Model* model)
+   : ImageState(OS::display->GetImage(OS::Display::ledImageIndex),false)
   {
-    // no code
+    SetModel(model);
+  }
+
+  LED* LED::Create()
+  {
+    LED *l;
+
+    l=new LED();
+
+    return l;
+  }
+
+  LED* LED::Create(Base::Model* model)
+  {
+    LED *l;
+
+    l=new LED();
+    l->SetModel(model);
+
+    return l;
   }
 }
