@@ -1925,6 +1925,16 @@ namespace Lum {
         return 1;
       }
 
+      size_t DefaultTheme::GetIconWidth() const
+      {
+        return 16;
+      }
+      
+      size_t DefaultTheme::GetIconHeight() const
+      {
+        return 16;
+      }
+
       size_t DefaultTheme::GetDragStartSensitivity() const
       {
         return 6;
