@@ -19,11 +19,11 @@
 
 #include <Lum/OS/Cairo/Font.h>
 
-#include <math.h>
+#include <cmath>
 
 #include <Lum/Base/String.h>
 
-#include <Lum/OS/Cairo/Display.h>
+#include <Lum/OS/Cairo/Driver.h>
 
 #if defined(CAIRO_HAS_WIN32_SURFACE)
   #include <cairo-win32.h>
@@ -35,14 +35,14 @@ namespace Lum {
 
       Font::Font()
       {
-        for (size_t x=0; x<maxStyleNum; x++) {
+        for (size_t x=0; x<sizeof(styles)/sizeof(FontStyle*); x++) {
           styles[x]=NULL;
         }
       }
 
       Font::~Font()
       {
-        for (size_t x=0; x<maxStyleNum; x++) {
+        for (size_t x=0; x<sizeof(styles)/sizeof(FontStyle*); x++) {
           if (styles[x]!=NULL) {
             if (styles[x]->fontFace!=NULL) {
               cairo_font_face_destroy(styles[x]->fontFace);
@@ -138,7 +138,9 @@ namespace Lum {
         cairo_font_options_t *options;
         cairo_font_extents_t fextents;
 
-        cairo_matrix_init_scale(&scaleMatrix,pixelHeight*::Lum::OS::display->GetDPI()/100,pixelHeight*::Lum::OS::display->GetDPI()/100);
+        cairo_matrix_init_scale(&scaleMatrix,
+                                pixelHeight*display->GetDPI()/100,
+                                pixelHeight*display->GetDPI()/100);
         cairo_get_matrix(cairo,&transformMatrix);
         options=cairo_font_options_create();
         cairo_surface_get_font_options(surface,options);
