@@ -113,8 +113,8 @@ namespace Lum {
         draw->PushFont(OS::display->GetFont());
         draw->PushForeground(Display::textColor);
 
-        for (int i=x; i<x+w; ++i) {
-          for (int j=y; j<y+h; ++j) {
+        for (int i=x; i<(int)(x+w); ++i) {
+          for (int j=y; j<(int)(y+h); ++j) {
 
             switch (image) {
             case Display::menuCheckImageIndex:
@@ -188,7 +188,7 @@ namespace Lum {
               if (i==x) {
                 draw->DrawFillString(i,j,L"[",OS::display->GetColor(Display::backgroundColor));
               }
-              else if (i==x+w-1) {
+              else if (i==(int)(x+w-1)) {
                 draw->DrawFillString(i,j,L"]",OS::display->GetColor(Display::backgroundColor));
               }
               else {
@@ -309,8 +309,8 @@ namespace Lum {
         draw->PushFont(OS::display->GetFont());
         draw->PushForeground(Display::textColor);
 
-        for (int i=x; i<x+w; ++i) {
-          for (int j=y; j<y+h; ++j) {
+        for (int i=x; i<(int)(x+w); ++i) {
+          for (int j=y; j<(int)(y+h); ++j) {
 
             switch (image) {
             case Theme::imageClose:
@@ -1019,6 +1019,16 @@ namespace Lum {
         return 1;
       }
 
+      size_t Theme::GetIconWidth() const
+      {
+        return 1;
+      }
+      
+      size_t Theme::GetIconHeight() const
+      {
+        return 1;
+      }
+
       size_t Theme::GetDragStartSensitivity() const
       {
         return 1;
