@@ -19,11 +19,19 @@
 
 #include <Lum/OS/Win32/Window.h>
 
+#include <iostream>
+
+#include <Lum/Private/Config.h>
+
 #include <Lum/OS/Win32/Display.h>
+#include <Lum/OS/Win32/DrawInfo.h>
 #include <Lum/OS/Win32/Event.h>
 #include <Lum/OS/Win32/Tray.h>
 
-#include <iostream>
+#if defined(LUM_HAVE_LIB_CAIRO)
+  #include <Lum/OS/Cairo/Display.h>
+  #include <Lum/OS/Cairo/DrawInfo.h>
+#endif
 
 namespace Lum {
   namespace OS {
@@ -271,6 +279,7 @@ namespace Lum {
       Window::Window(OS::Display* display)
       : display(dynamic_cast<Display*>(display)),
         drawn(false),queueFocus(false),
+        paintDraw(NULL),
         window(0)
       {
         // no code
@@ -451,10 +460,14 @@ namespace Lum {
         }
 
         SetTitle(GetTitle());
+        
+        {
+          HRGN region;
 
-        draw=OS::driver->CreateDrawInfo(this);
-        draw->PushClip(0,0,display->GetScreenWidth(),display->GetScreenHeight());
-
+          region=CreateRectRgn(0,0,rect.right-rect.left+1,rect.bottom-rect.top+1);
+          /* ignore */ SelectClipRgn(GetDC(window),region);
+        }
+        
         if ((type==typeMenu ||
              type==typeMenuDropdown ||
              type==typeMenuPopup ||
@@ -523,12 +536,6 @@ namespace Lum {
           }
         }
 
-        if (draw!=NULL) {
-          draw->PopClip();
-          delete draw;
-          draw=NULL;
-        }
-
         dynamic_cast<Display*>(display)->RemoveWindow(this);
 
         OS::Base::Window::Close();
@@ -581,17 +588,116 @@ namespace Lum {
         }
       }
 
+      void Window::TriggerDraw(int x, int y, size_t width, size_t height)
+      {
+        RECT rect;
+        
+        ::GetClientRect(window,&rect);
+        
+        rect.left=x;
+        rect.top=y;
+        rect.right=x+width;
+        rect.bottom=y+height;
+        
+        ::InvalidateRect(window,&rect,true);
+      }
+      
       bool Window::SendBalloonMessage(const std::wstring& /*text*/,
                                       unsigned long /*timeout*/)
-
       {
         return true;
       }
 
-      LRESULT Window::MessageHandler(HWND wnd,UINT msg, WPARAM wParam, LPARAM lParam)
+      void Window::OnPaint()
       {
         HDC          dc;
         PAINTSTRUCT  ps;
+        OS::Bitmap   *bitmap=NULL;
+        OS::DrawInfo *bitmapDraw=NULL;
+        DrawMsg      msg;
+        
+        if (IsShown() && !onOpened) {
+          TriggerPreOpened();
+        }
+      
+        dc=::BeginPaint(window,&ps);
+    
+#if defined(CAIRO_HAS_WIN32_SURFACE)
+        if (driver->GetDriverName()==L"CairoWin32") {
+          paintDraw=new Cairo::Win32DrawInfo(this,dc);
+        }
+        else {
+          paintDraw=new DrawInfo(this,dc);
+        }
+#else          
+        paintDraw=new DrawInfo(this,dc);
+#endif  
+
+        msg.x=ps.rcPaint.left;
+        msg.y=ps.rcPaint.top;
+        msg.width=ps.rcPaint.right-ps.rcPaint.left;
+        msg.height=ps.rcPaint.bottom-ps.rcPaint.top;
+
+        bitmap=driver->CreateBitmap(GetWidth(),GetHeight());
+
+        if (bitmap!=NULL) {
+          bitmapDraw=bitmap->GetDrawInfo();
+
+          if (bitmapDraw==NULL) {
+            delete bitmap;
+            bitmap=NULL;
+          }
+          else {
+            paintDraw->CopyToBitmap(msg.x,msg.y,msg.width,msg.height,msg.x,msg.y,bitmap);
+          }
+        }
+
+        if (bitmapDraw!=NULL) {
+          msg.draw=bitmapDraw;
+        }
+        else {
+          msg.draw=paintDraw;
+        }
+
+        paintDraw->PushClip(msg.x,msg.y,msg.width,msg.height);
+        
+        drawAction->Trigger(msg);
+      
+        paintDraw->PopClip();
+      
+        if (bitmap!=NULL) {
+          paintDraw->CopyFromBitmap(bitmap,msg.x,msg.y,msg.width,msg.height,msg.x,msg.y);
+
+          delete bitmap;
+          bitmap=NULL;
+          bitmapDraw=NULL;
+        }
+      
+        delete paintDraw;
+        paintDraw=NULL;
+    
+        /* ignore */ ::EndPaint(window,&ps);
+
+        if (IsShown() && !onOpened) {
+          TriggerOpened();
+        }
+        
+        /*
+          HACK: If we received focus before our window is
+          drawn, send one focus message after first drawing
+          the window
+         */
+        drawn=true;
+
+        // See WM_SETFOCUS handling
+        if (queueFocus || grabWindow==this) {
+          TriggerFocusIn();
+          queueFocus=false;
+        }
+      }
+      
+      LRESULT Window::MessageHandler(HWND wnd,UINT msg, WPARAM wParam, LPARAM lParam)
+      {
         OS::EventRef event;
         MSG          winMsg;
         WINDOWPOS    *pos;
@@ -602,29 +708,7 @@ namespace Lum {
           TriggerClosed();
           return 0;
         case WM_PAINT:
-          dc=::BeginPaint(wnd,&ps);
-          draw->PushClip(ps.rcPaint.left,ps.rcPaint.top,
-                         ps.rcPaint.right-ps.rcPaint.left+1,
-                         ps.rcPaint.bottom-ps.rcPaint.top+1);
-          TriggerRedraw(ps.rcPaint.left,ps.rcPaint.top,
-                        ps.rcPaint.right-ps.rcPaint.left+1,
-                        ps.rcPaint.bottom-ps.rcPaint.top+1);
-          draw->PopClip();
-          /* ignore */ ::EndPaint(wnd,&ps);
-
-          /*
-            HACK: If we received focus before our window is
-            drawn, send one focus message after first drawing
-            the window
-           */
-          drawn=true;
-
-          // See WM_SETFOCUS handling
-          if (queueFocus || grabWindow==this) {
-            TriggerFocusIn();
-            queueFocus=false;
-          }
-
+          OnPaint();
           break;
         case WM_SIZE:
           if (wParam==SIZE_MINIMIZED) {
@@ -722,9 +806,6 @@ namespace Lum {
           if (pos->cx-wDelta!=(int)width || pos->cy-hDelta!=(int)height) {
             width=pos->cx-wDelta; // TODO: check width/height/clip rect
             height=pos->cy-hDelta;
-            draw->PopClip();
-            draw->SetNewSize(width,height);
-            draw->PushClip(0,0,width,height);
             TriggerResize(width,height);
           }
           break;
