@@ -94,6 +94,16 @@ namespace Lum {
   }
 
 
+  const wchar_t* Object::inputTypeAlpha                     = L"Alpha";
+  const wchar_t* Object::inputTypeUnsignedBinaryNumber      = L"UnsignedBinaryNumber";
+  const wchar_t* Object::inputTypeUnsignedOctalNumber       = L"UnsignedOctalNumber";
+  const wchar_t* Object::inputTypeUnsignedHexadecimalNumber = L"UnsignedHexadecimalNumber";
+  const wchar_t* Object::inputTypeSignedDecimalNumber       = L"SignedDecimalNumber";
+  const wchar_t* Object::inputTypeUnsignedDecimalNumber     = L"UnsignedDecimalNumber";
+  const wchar_t* Object::inputTypeSignedFloatNumber         = L"SsignedFloatNumber";
+  const wchar_t* Object::inputTypeUnsignedFloatNumber       = L"UnsignedFloatNumber";
+  const wchar_t* Object::inputTypePhoneNumber               = L"PhoneNumber";
+
   Object::Object()
   : parent(NULL),
     x(0),y(0),width(0),height(0),
@@ -107,6 +117,7 @@ namespace Lum {
     firstDraw(true),
     layouted(false),
     clipFromParent(false),
+    clipBackgroundToBorder(false),
     hasFocus(false),
     canFocus(false),
     showFocus(false),
@@ -574,25 +585,22 @@ namespace Lum {
     window=NULL;
   }
 
+  /**
+    Return the parent object (or NULL, if this object is root) in the object tree.
+    */
   Object* Object::GetParent() const
   {
     return parent;
   }
 
   /**
-    \return Returns the OS::DrawInfo used for all drawing.
-  */
-  OS::DrawInfo* Object::GetDrawInfo() const
-  {
-    assert(GetWindow()!=NULL);
-
-    return GetWindow()->GetDrawInfo();
-  }
-
-  OS::Color Object::GetTextColor(OS::DrawInfo *draw) const
+    Return the text color to be used for text drawing in this control and in children
+    */
+  OS::Color Object::GetTextColor(OS::DrawInfo* draw,
+                                 const void* child) const
   {
     if (parent!=NULL) {
-      return parent->GetTextColor(draw);
+      return parent->GetTextColor(draw, this);
     }
     else if (draw->disabled) {
       return OS::display->GetColor(OS::Display::textDisabledColor);
@@ -605,6 +613,46 @@ namespace Lum {
     }
   }
 
+  /**
+    Return the font to be used for text drawing in this control and in children
+    */
+  OS::Font* Object::GetFont(const void* child) const
+  {
+    if (parent!=NULL) {
+      return parent->GetFont(this);
+    }
+    else {
+      return OS::display->GetFont();
+    }
+  }
+
+  /**
+    Return the font style to be used for text drawing in this control and in children
+    */
+  unsigned long Object::GetFontStyle(const void* child) const
+  {
+    if (parent!=NULL) {
+      return parent->GetFontStyle(this);
+    }
+    else {
+      return OS::Font::normal;
+    }
+  }
+
+  /**
+    If true, text in this control and for children should be drawn "smart",
+    that means in pseudo 3D look.
+    */
+  bool Object::GetFontStyleSmart(const void* child) const
+  {
+    if (parent!=NULL) {
+      return parent->GetFontStyleSmart(this);
+    }
+    else {
+      return false;
+    }
+  }
+
   /**
     Assign a OS::Window that will displayed as tooltip. The base class will
     store the object within a private variable and will return it when
@@ -661,9 +709,12 @@ namespace Lum {
     // No code - the default implementation does nothing
   }
 
-  bool Object::DisplayFocus() const
+  /**
+    Returns true, if the object does not have the focus but still should show the focus
+    */
+  bool Object::ShowFocus() const
   {
-    return hasFocus || showFocus;
+    return showFocus;
   }
 
   /**
@@ -683,11 +734,20 @@ namespace Lum {
   }
 
   /**
-    \return Returns true, when the object requests focus handling and is visible.
+    \return Returns true, if the object supports focus handling and is visible.
   */
-  bool Object::HandlesFocus() const
+  bool Object::HandlesClickFocus() const
   {
-    return requestsFocus && visible;
+    return CanFocus() && IsVisible();
+  }
+
+  /**
+    \return Returns true, when the object requests focus handling as part of a
+    focus chain and is visible.
+  */
+  bool Object::HandlesKeyFocus() const
+  {
+    return RequestedFocus() && IsVisible();
   }
 
   void Object::SetCanFocus(bool can)
@@ -723,7 +783,7 @@ namespace Lum {
   }
 
   /**
-    \return Returns true, when the object supports focusing.
+    \return Returns true, if the object actually has requested to handle focus.
   */
   bool Object::RequestedFocus() const
   {
@@ -769,6 +829,18 @@ namespace Lum {
     return requestsKeyboard;
   }
 
+  Object* Object::SetInputTypeHint(const std::wstring& inputTypeHint)
+  {
+    this->inputTypeHint=inputTypeHint;
+
+    return this;
+  }
+
+  std::wstring Object::GetInputTypeHint() const
+  {
+    return inputTypeHint;
+  }
+
   IMHandler* Object::GetIMHandler()
   {
     return NULL;
@@ -782,7 +854,7 @@ namespace Lum {
   public:
     virtual bool Visit(Object* object)
     {
-      if (object->HandlesFocus()) {
+      if (object->HandlesKeyFocus()) {
         this->object=object;
         return false;
       }
@@ -808,7 +880,7 @@ namespace Lum {
       object->VisitChildren(*this,true);
 
       if (this->object==NULL) {
-        if (object->HandlesFocus()) {
+        if (object->HandlesKeyFocus()) {
           this->object=object;
         }
       }
@@ -950,7 +1022,7 @@ namespace Lum {
   {
     FocusFirstVisitor visitor;
 
-    if (HandlesFocus()) {
+    if (HandlesKeyFocus()) {
       return this;
     }
 
@@ -977,7 +1049,7 @@ namespace Lum {
       return visitor.object;
     }
 
-    if (HandlesFocus()) {
+    if (HandlesKeyFocus()) {
       return this;
     }
     else {
@@ -1026,7 +1098,7 @@ namespace Lum {
       }
 
       /*
-        If we a called by a child and connot find a follower within our
+        If we are called by a child and connot find a follower within our
         list of children we delegate it to the parent to investigate all
         other subtrees.
       */
@@ -1043,11 +1115,11 @@ namespace Lum {
       }
 
       // Now, if that fails: can we handle it ourself?
-      if (HandlesFocus()) {
+      if (HandlesKeyFocus()) {
         return this;
       }
 
-      // Search for the next object in our list of children
+      // Search for the previous object in our list of children
       before.start=true;
       before.base=object;
       before.object=NULL;
@@ -1099,7 +1171,7 @@ namespace Lum {
       }
 
       // Now, if that fails: can we handle it ourself?
-      if (HandlesFocus()) {
+      if (HandlesKeyFocus()) {
         return this;
       }
 
@@ -1478,8 +1550,8 @@ namespace Lum {
 
     LayoutVisitor layoutVisitor;
 
+    VisitChildren(layoutVisitor,true);
     layouted=true;
-    layoutVisitor.Visit(this);
   }
 
   /**
@@ -1744,11 +1816,24 @@ namespace Lum {
 
             object->PrepareForBackground(draw);
 
+            if (object->clipBackgroundToBorder) {
+              draw->PushClipBegin(object->GetOX(),object->GetOY(),
+                                  object->GetOWidth(),object->GetOHeight());
+              draw->SubClipRegion(object->GetX(),object->GetY(),
+                                  object->GetWidth(),object->GetHeight());
+              draw->PushClipEnd();
+
+            }
+
             object->background->Draw(draw,
                                      object->GetOX(),object->GetOY(),
                                      object->GetOWidth(),object->GetOHeight(),
                                      cx,cy,cw,ch);
 
+            if (object->clipBackgroundToBorder) {
+              draw->PopClip();
+            }
+
             draw->activated=false;
             draw->selected=false;
             draw->disabled=false;
@@ -1825,7 +1910,8 @@ namespace Lum {
       a clipping reagion might be active. Use OS::display->QueueActionForEventLoop() for this</li>
     </ul>
   */
-  void Object::Draw(int x, int y, size_t w, size_t h)
+  void Object::Draw(OS::DrawInfo* draw,
+                    int x, int y, size_t w, size_t h)
   {
     visible=true;
     firstDraw=false;
@@ -1837,12 +1923,13 @@ namespace Lum {
     /* --- */
 
     // Normally everything should be layouted in the layouting phase, however
-    // not all objetc implement VisitChildren or call Layout manually for their children.
+    // not all objects implement VisitChildren() or call Layout() manually for
+    // their children.
     if (!IsLayouted()) {
       Layout();
     }
 
-    DrawChildren(GetDrawInfo(),x,y,w,h);
+    DrawChildren(draw,x,y,w,h);
   }
 
   void Object::DrawChildren(OS::DrawInfo* draw, int x, int y, size_t w, size_t h)
@@ -1925,7 +2012,7 @@ namespace Lum {
         }
 
         group->PreDrawChild(draw,object);
-        object->Draw(x,y,width,height);
+        object->Draw(draw,x,y,width,height);
         group->PostDrawChild(draw,object);
 
         return true;
@@ -1953,14 +2040,14 @@ namespace Lum {
   void Object::Redraw()
   {
     if (visible) {
-      GetWindow()->TriggerRedraw(oX,oY,oWidth,oHeight);
+      GetWindow()->TriggerDraw(oX,oY,oWidth,oHeight);
     }
   }
 
   void Object::Redraw(int x, int y, size_t w, size_t h)
   {
     if (visible) {
-      GetWindow()->TriggerRedraw(x,y,w,h);
+      GetWindow()->TriggerDraw(x,y,w,h);
     }
   }
 
@@ -1988,8 +2075,6 @@ namespace Lum {
   {
     class HideVisitor : public Visitor
     {
-    public:
-      OS::DrawInfo *draw;
 
       bool Visit(Object* object)
       {
@@ -1998,20 +2083,20 @@ namespace Lum {
       }
     };
 
-    if (visible) {
-      HideVisitor visitor;
+    if (!visible) {
+      return;
+    }
 
-      visitor.draw=GetDrawInfo();
+    HideVisitor visitor;
 
-      VisitChildren(visitor,true);
+    VisitChildren(visitor,true);
 
-      visible=false;
+    visible=false;
 
-      // Tell the window, that this object hides
-      // Dialog may delgate focus or other things
-      if (GetWindow()!=NULL && dynamic_cast<Window*>(GetWindow()->GetMaster())!=NULL) {
-        dynamic_cast<Window*>(GetWindow()->GetMaster())->ObjectHides(this);
-      }
+    // Tell the window, that this object hides
+    // Dialog may delegate focus or other things
+    if (GetWindow()!=NULL && dynamic_cast<Window*>(GetWindow()->GetMaster())!=NULL) {
+      dynamic_cast<Window*>(GetWindow()->GetMaster())->ObjectHides(this);
     }
   }
 
@@ -2061,6 +2146,11 @@ namespace Lum {
     clipFromParent=clip;
   }
 
+  void Object::SetClipBackgroundToBorder(bool clip)
+  {
+    clipBackgroundToBorder=clip;
+  }
+
   /**
     Called when the mouse enters the object
   */
@@ -2202,9 +2292,19 @@ namespace Lum {
     // no code
   }
 
-  bool Control::HandlesFocus() const
+  bool Control::HandlesClickFocus() const
   {
-    return Object::HandlesFocus() && model.Valid() && model->IsEnabled();
+    return CanFocus() &&
+           IsVisible() &&
+           model.Valid();
+  }
+
+  bool Control::HandlesKeyFocus() const
+  {
+    return HandlesClickFocus() &&
+           RequestedFocus() &&
+           model.Valid() &&
+           model->IsEnabled();
   }
 
   /**
@@ -2233,13 +2333,23 @@ namespace Lum {
     return model.Get();
   }
 
+  bool Control::HasModel() const
+  {
+    return model.Valid();
+  }
+
   void Control::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
     // Delegate the focus, if we get disabled
-    if (model==this->model && !model->IsEnabled() && HasFocus()) {
+    if (model==this->model &&
+        this->model.Valid() &&
+        !model->IsEnabled() &&
+        HasFocus() &&
+        HandlesKeyFocus()) {
       Window* dialog;
 
-      if (GetWindow()!=NULL && (dialog=dynamic_cast<Window*>(GetWindow()->GetMaster()))!=NULL) {
+      if (GetWindow()!=NULL &&
+          (dialog=dynamic_cast<Window*>(GetWindow()->GetMaster()))!=NULL) {
         dialog->FocusNext();
       }
     }
@@ -2250,12 +2360,10 @@ namespace Lum {
   Scrollable::Scrollable()
   : hAdjustment(new Model::Adjustment()),
     vAdjustment(new Model::Adjustment()),
-    updateDimensionsAction(new Model::Action())
+    scrollView(NULL)
   {
     Observe(hAdjustment->GetTopModel());
     Observe(vAdjustment->GetTopModel());
-
-    Observe(updateDimensionsAction);
   }
 
   Scrollable::~Scrollable()
@@ -2263,6 +2371,21 @@ namespace Lum {
     // no code
   }
 
+  void Scrollable::GetODimension(size_t& width, size_t& height)
+  {
+    if (background.Valid()) {
+      width-=background->leftBorder+background->rightBorder;
+      height-=background->topBorder+background->bottomBorder;
+    }
+
+    GetDimension(width,height);
+
+    if (background.Valid()) {
+      width+=background->leftBorder+background->rightBorder;
+      height+=background->topBorder+background->bottomBorder;
+    }
+  }
+
   /**
     If true returned, this object requires scroll knobs to change the
     adjustment. If false it returned, it is assumed that the widget itself has ways to change
@@ -2306,28 +2429,28 @@ namespace Lum {
     return vAdjustment;
   }
 
-  void Scrollable::Layout()
+  void Scrollable::SetScrollView(Object *scrollView)
   {
-    UpdateDimensions();
-
-    Control::Layout();
+     this->scrollView=scrollView;
   }
 
-  void Scrollable::Draw(int x, int y, size_t w, size_t h)
+  void Scrollable::Redraw()
   {
-    Control::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
+    if (scrollView!=NULL) {
+      scrollView->Redraw();
+    }
+    else {
+      Object::Redraw();
     }
-
-    OS::display->QueueActionForEventLoop(updateDimensionsAction);
   }
-  void Scrollable::Resync(Base::Model* model, const Base::ResyncMsg& msg)
+
+  void Scrollable::Redraw(int x, int y, size_t w, size_t h)
   {
-    if (model==updateDimensionsAction &&
-        updateDimensionsAction->IsFinished()) {
-      UpdateDimensions();
+    if (scrollView!=NULL) {
+      scrollView->Redraw(x,y,w,h);
+    }
+    else {
+      Object::Redraw(x,y,w,h);
     }
   }
 
@@ -2428,7 +2551,6 @@ namespace Lum {
     assert(container!=NULL);
 
     container->MoveResize(x,y,width,height);
-    container->Layout();
 
     Object::Layout();
   }
@@ -2490,7 +2612,6 @@ namespace Lum {
   void ControlComponent::Layout()
   {
     container->MoveResize(x,y,width,height);
-    container->Layout();
 
     Control::Layout();
   }
