@@ -104,10 +104,9 @@ namespace Lum {
     boxFrame->topBorder+((int)knob->GetHeight()-(int)font->height)/2+font->ascent;
   }
 
-  void HSlider::DrawKnob()
+  void HSlider::DrawKnob(OS::DrawInfo* draw)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-    int          boxSize,kStart,offset;
+    int boxSize,kStart,offset;
 
     draw->activated=IsMouseActive();
     draw->focused=HasFocus();
@@ -208,14 +207,14 @@ namespace Lum {
     Control::Layout();
   }
 
-  void HSlider::Draw(int x, int y, size_t w, size_t h)
+  void HSlider::Draw(OS::DrawInfo* draw,
+                     int x, int y, size_t w, size_t h)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-    size_t       fOffset,fSize;
+    size_t fOffset,fSize;
 
     draw->focused=HasFocus();
 
-    Control::Draw(x,y,w,h);
+    Control::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       draw->focused=false;
@@ -233,14 +232,14 @@ namespace Lum {
     fSize=knob->GetHeight()+boxFrame->minHeight;
 
     if (useScale) {
-      scale->Draw(x,y,w,h);
+      scale->Draw(draw,x,y,w,h);
     }
 
     draw->focused=false;
 
     boxFrame->Draw(draw,this->x,this->y+fOffset,this->width,fSize);
 
-    DrawKnob();
+    DrawKnob(draw);
   }
 
   void HSlider::HandleMouseMovement(int x, int /*y*/)
@@ -251,7 +250,10 @@ namespace Lum {
       x=this->x+boxFrame->leftBorder+corr;
     }
 
-    newValue=((x-(this->x+boxFrame->leftBorder)-corr)*(model->GetMax()-model->GetMin()+1))/(width-boxFrame->minWidth-knob->GetWidth());
+    newValue=model->GetMin()+
+        ((x-(this->x+boxFrame->leftBorder)-corr)*
+            (model->GetMax()-model->GetMin()+1))/
+        (width-boxFrame->minWidth-knob->GetWidth());
 
     if (newValue>=model->GetMin() && newValue<=model->GetMax()) {
       model->Set(newValue);
@@ -272,17 +274,19 @@ namespace Lum {
 
     if (event.type==OS::MouseEvent::down && PointIsIn(event)) {
       if (event.button==OS::MouseEvent::button1) {
-        if (event.x>=knobX && event.y>=knobY && event.x<knobX+knobWidth && event.y<knobY+knobHeight) {
+        if (event.x>=knobX && event.y>=knobY &&
+            event.x<(int)(knobX+knobWidth) &&
+            event.y<(int)(knobY+knobHeight)) {
           corr=event.x-knobX;
           selected=true;
-          DrawKnob();
+          Redraw(); // TODO: ONly knob
 
           OnMoveStart();
         }
         else if (event.x<knobX) {
           Dec();
         }
-        else if (event.x>knobX+knobWidth) {
+        else if (event.x>(int)(knobX+knobWidth)) {
           Inc();
         }
       }
@@ -301,7 +305,7 @@ namespace Lum {
         selected=false;
       }
 
-      DrawKnob();
+      Redraw(); // TODO: Only knob
 
       OnMoveEnd();
 
@@ -401,10 +405,9 @@ namespace Lum {
     boxFrame=OS::display->GetFrame(OS::Display::hscaleFrameIndex);
   }
 
-  void VSlider::DrawKnob()
+  void VSlider::DrawKnob(OS::DrawInfo* draw)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-    int          boxSize,kStart,offset;
+    int boxSize,kStart,offset;
 
     draw->activated=IsMouseActive();
     draw->focused=HasFocus();
@@ -504,14 +507,14 @@ namespace Lum {
     Control::Layout();
   }
 
-  void VSlider::Draw(int x, int y, size_t w, size_t h)
+  void VSlider::Draw(OS::DrawInfo* draw,
+                     int x, int y, size_t w, size_t h)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-    size_t       fOffset,fSize;
+    size_t fOffset,fSize;
 
     draw->focused=HasFocus();
 
-    Control::Draw(x,y,w,h);
+    Control::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       draw->focused=false;
@@ -529,14 +532,14 @@ namespace Lum {
     fSize=knob->GetWidth()+boxFrame->minWidth;
 
     if (useScale) {
-      scale->Draw(x,y,w,h);
+      scale->Draw(draw,x,y,w,h);
     }
 
     boxFrame->Draw(draw,this->x+fOffset,this->y,fSize,this->height);
 
     draw->focused=false;
 
-    DrawKnob();
+    DrawKnob(draw);
   }
 
   void VSlider::HandleMouseMovement(int /*x*/, int y)
@@ -568,17 +571,19 @@ namespace Lum {
 
     if (event.type==OS::MouseEvent::down && PointIsIn(event)) {
       if (event.button==OS::MouseEvent::button1) {
-        if (event.x>=knobX && event.y>=knobY && event.x<knobX+knobWidth && event.y<knobY+knobHeight) {
+        if (event.x>=knobX && event.y>=knobY &&
+            event.x<(int)(knobX+knobWidth) &&
+            event.y<(int)(knobY+knobHeight)) {
           corr=event.y-knobY;
           selected=true;
-          DrawKnob();
+          Redraw(); // TODO: Only knob
 
           OnMoveStart();
         }
         else if (event.y<knobY) {
           Dec();
         }
-        else if (event.y>knobY+knobHeight) {
+        else if (event.y>(int)(knobY+knobHeight)) {
           Inc();
         }
       }
@@ -597,7 +602,7 @@ namespace Lum {
         selected=false;
       }
 
-      DrawKnob();
+      Redraw(); // TODO: Only knob
 
       OnMoveEnd();
 
