@@ -19,11 +19,15 @@
 
 #include <Lum/String.h>
 
-#include <Lum/OS/Manager/Repository.h>
-#include <Lum/OS/Theme.h>
-
 #include <Lum/Base/String.h>
 
+#include <Lum/Manager/Keyboard.h>
+
+#include <Lum/Model/Number.h>
+#include <Lum/Model/String.h>
+
+#include <Lum/OS/Theme.h>
+
 #include <Lum/Array.h>
 #include <Lum/Button.h>
 #include <Lum/Menu.h>
@@ -49,31 +53,14 @@ namespace Lum {
       data=L"";
       cursorOffset=0;
 
-      Model::StringRef model=dynamic_cast<Model::String*>(control->GetModel());
-
-      if (!model.Valid() ||
-          model->IsNull() ||
-          !model->IsEnabled()) {
-        return;
-      }
-
-      data=model->Get();
-      cursorOffset=control->GetCursorPos();
+      control->GetBuffer(data,cursorOffset);
     }
 
     void SetData(const std::wstring& data)
     {
       assert(control!=NULL);
 
-      Model::StringRef model=dynamic_cast<Model::String*>(control->GetModel());
-
-      if (!model.Valid() ||
-          model->IsNull() ||
-          !model->IsEnabled()) {
-        return;
-      }
-
-      model->Set(data);
+      control->SetBuffer(data);
     }
 
     void SetCursorOffset(size_t cursorOffset)
@@ -83,8 +70,7 @@ namespace Lum {
       Model::StringRef model=dynamic_cast<Model::String*>(control->GetModel());
 
       if (!model.Valid() ||
-          model->IsNull() ||
-          !model->IsEnabled()) {
+          model->IsNull()) {
         return;
       }
 
@@ -197,14 +183,15 @@ namespace Lum {
   }
 
   String::String()
-  : textAlign(left),textWidth(256),baseline(false),
+  : textAlign(left),
+    baseline(false),
     visWidth(std::max(20u,OS::display->GetFont()->height*5)),
     offset(0),cursor(0),
     markA((size_t)-1),markB((size_t)-1),
     font(OS::display->GetFont()),
     model(NULL),
+    sync(true),
     selected(false),
-    readOnly(false),
     cursorVisible(false),
     showClearButton(false),
     clearImage(OS::display->GetTheme()->GetStockImage(OS::Theme::imageClear)),
@@ -214,8 +201,7 @@ namespace Lum {
     copyAction(new Model::Action),
     pasteAction(new Model::Action),
     eraseAction(new Model::Action),
-    markAllAction(new Model::Action),
-    inputHelper(NULL)
+    markAllAction(new Model::Action)
   {
     SetBackground(OS::display->GetFill(OS::Display::entryBackgroundFillIndex));
 
@@ -243,8 +229,6 @@ namespace Lum {
   String::~String()
   {
     OS::display->RemoveTimer(blinkAction);
-
-    delete inputHelper;
   }
 
   bool String::HasBaseline() const
@@ -299,13 +283,43 @@ namespace Lum {
 
   bool String::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::String*>(model);
+    this->model=model;
 
     Control::SetModel(this->model);
 
+    if (!converter.Valid()) {
+      if (dynamic_cast<Model::String*>(model)) {
+        SetConverter(new Model::StringStringConverter());
+      }
+      else if (dynamic_cast<Model::Int*>(model)) {
+        SetConverter(new Model::IntStringConverter());
+      }
+      else if (dynamic_cast<Model::UInt*>(model)) {
+        SetConverter(new Model::UIntStringConverter());
+      }
+      else if (dynamic_cast<Model::Long*>(model)) {
+        SetConverter(new Model::LongStringConverter());
+      }
+      else if (dynamic_cast<Model::ULong*>(model)) {
+        SetConverter(new Model::ULongStringConverter());
+      }
+      else if (dynamic_cast<Model::SizeT*>(model)) {
+        SetConverter(new Model::SizeTStringConverter());
+      }
+    }
+
     return this->model.Valid();
   }
 
+  String* String::SetConverter(Model::StringConverter* converter)
+  {
+    this->converter=converter;
+
+    SyncModelToBuffer();
+
+    return this;
+  }
+
   String* String::SetLabel(const std::wstring& label)
   {
     this->label=label;
@@ -315,29 +329,53 @@ namespace Lum {
     return this;
   }
 
-  String* String::SetInputHelper(InputHelper *inputHelper)
+  void String::SyncModelToBuffer()
   {
-    delete this->inputHelper;
-
-    this->inputHelper=inputHelper;
-
-    SetRequestsKeyboard(this->inputHelper==NULL);
-
-    return this;
+    if (converter.Valid()) {
+      converter->ConvertModelToString(model,buffer);
+    }
   }
 
-  void String::OpenInputHelper()
+  void String::SyncBufferToModel()
   {
-    if (OS::Manager::repository->GetKeyboardManager()!=NULL &&
-        inputHelper!=NULL &&
-        !inputHelper->IsOpen()) {
-      inputHelper->SetType(OS::Window::typePopup);
-      inputHelper->SetParent(GetWindow());
-      inputHelper->SetReference(this);
-      inputHelper->Open(this);
+    if (converter.Valid()) {
+      sync=false;
+      converter->ConvertStringToModel(buffer,model);
+      sync=true;
+
+      Redraw();
     }
   }
 
+  bool String::GetBuffer(std::wstring& buffer,
+                         size_t& cursorOffset) const
+  {
+    if (!model.Valid()) {
+      return false;
+    }
+
+    buffer=this->buffer;
+    cursorOffset=this->cursor;
+
+    return true;
+  }
+
+  bool String::SetBuffer(const std::wstring& buffer)
+  {
+    if (!model.Valid() ||
+        model->IsDisabled()) {
+      return false;
+    }
+
+    this->buffer=buffer;
+
+    SyncBufferToModel();
+
+    Redraw();
+
+    return true;
+  }
+
   void String::SetCursorUpAction(Model::Action* action)
   {
     cursorUpAction=action;
@@ -443,56 +481,71 @@ namespace Lum {
 
   void String::ActionInsertString(const std::wstring& string)
   {
-    if (model->Get().length()+string.length()<=textWidth) {
-      DeleteSelection();
-      cursor+=string.length();
-      model->Insert(cursor-string.length(),string);
-    }
-    else {
+    if (!model.Valid() ||
+        !model->IsEnabled()) {
       OS::display->Beep();
+
+      return;
     }
+
+    DeleteSelection();
+    buffer.insert(cursor,string);
+    cursor+=string.length();
+
+    SyncBufferToModel();
   }
 
   void String::ActionCut()
   {
-    std::wstring text;
+    if (!model.Valid() ||
+        !Selected() ||
+        !model->IsEnabled()) {
+      OS::display->Beep();
 
-    if (model.Valid() && Selected()) {
-      text=model->Get();
-      text=text.substr(markA,markB-markA);
-      if (!OS::display->SetClipboard(text)) {
-        OS::display->Beep();
-      }
-      else {
-        DeleteSelection();
-      }
+      return;
     }
-    else {
+
+    std::wstring text(buffer);
+
+    text=text.substr(markA,markB-markA);
+
+    if (!OS::display->SetClipboard(text)) {
       OS::display->Beep();
     }
+
+    DeleteSelection();
   }
 
   void String::ActionCopy()
   {
-    std::wstring text;
+    if (!model.Valid() ||
+        !Selected()) {
+      OS::display->Beep();
 
-    if (model.Valid() && Selected()) {
-      text=model->Get();
-      text=text.substr(markA,markB-markA);
-      if (!OS::display->SetClipboard(text)) {
-        OS::display->Beep();
-      }
+      return;
     }
-    else {
+
+    std::wstring text(buffer);
+
+    text=text.substr(markA,markB-markA);
+
+    if (!OS::display->SetClipboard(text)) {
       OS::display->Beep();
     }
   }
 
   void String::ActionPaste()
   {
+    if (!model->IsEnabled()) {
+      OS::display->Beep();
+
+      return;
+    }
+
     std::wstring res;
 
     res=OS::display->GetClipboard();
+
     if (!res.empty()) {
       ActionInsertString(res);
     }
@@ -500,23 +553,31 @@ namespace Lum {
 
   void String::ActionErase()
   {
-    if (model.Valid() && Selected()) {
-      DeleteSelection();
-    }
-    else {
+    if (!model.Valid() ||
+        !Selected() ||
+        !model->IsEnabled()) {
       OS::display->Beep();
+
+      return;
     }
+
+    DeleteSelection();
   }
 
   void String::ActionEraseAll()
   {
-    if (model.Valid()) {
-      ClearSelection();
-      model->Set(L"");
-    }
-    else {
+    if (!model.Valid() ||
+        !model->IsEnabled()) {
       OS::display->Beep();
+
+      return;
     }
+
+    ClearSelection();
+
+    buffer=L"";
+
+    SyncBufferToModel();
   }
 
   void String::ActionSelectAll()
@@ -527,32 +588,34 @@ namespace Lum {
 
   bool String::ActionReturn()
   {
-    if (returnAction.Valid()) {
-      if (!HasFocus()) {
-        selected=false;
-        Redraw();
-      }
-      returnAction->Trigger();
-      return true;
-    }
-    else {
+    if (!returnAction.Valid()) {
       return false;
     }
+
+    if (!HasFocus()) {
+      selected=false;
+      Redraw();
+    }
+
+    returnAction->Trigger();
+
+    return true;
   }
 
   bool String::ActionEscape()
   {
-    if (escapeAction.Valid()) {
-      if (!HasFocus()) {
-        selected=false;
-        Redraw();
-      }
-      escapeAction->Trigger();
-      return true;
-    }
-    else {
+    if (!escapeAction.Valid()) {
       return false;
     }
+
+    if (!HasFocus()) {
+      selected=false;
+      Redraw();
+    }
+
+    escapeAction->Trigger();
+
+    return true;
   }
 
   void String::ActionSelectLeft()
@@ -562,15 +625,18 @@ namespace Lum {
         cursor--;
         SetSelection(cursor,cursor+1);
         Redraw();
-      } else if ((markA+1==markB) && (markB==cursor)) {
+      }
+      else if ((markA+1==markB) && (markB==cursor)) {
         ClearSelection();
         cursor--;
         Redraw();
-      } else if (cursor==markA) {
+      }
+      else if (cursor==markA) {
         cursor--;
         SetSelection(cursor,markB);
         Redraw();
-      } else if (cursor==markB) {
+      }
+      else if (cursor==markB) {
         cursor--;
         SetSelection(markA,cursor);
         Redraw();
@@ -580,7 +646,7 @@ namespace Lum {
 
   void String::ActionSelectRight()
   {
-    if (cursor<model->Get().length()) {
+    if (cursor<buffer.length()) {
       if (!Selected()) {
         cursor++;
         SetSelection(cursor-1,cursor);
@@ -590,11 +656,13 @@ namespace Lum {
         ClearSelection();
         cursor++;
         Redraw();
-      } else if (cursor==markB) {
+      }
+      else if (cursor==markB) {
         cursor++;
         SetSelection(markA,cursor);
         Redraw();
-      } else if  (cursor==markA) {
+      }
+      else if (cursor==markA) {
         cursor++;
         SetSelection(cursor,markB);
         Redraw();
@@ -607,12 +675,15 @@ namespace Lum {
     if (Selected()) {
       if (cursor==markA) {
         SetSelection(0,markB);
-      } else {
+      }
+      else {
         SetSelection(0,markA);
       }
-    } else {
+    }
+    else {
       SetSelection(0,cursor);
     }
+
     cursor=0;
     Redraw();
   }
@@ -621,38 +692,39 @@ namespace Lum {
   {
     if (Selected()) {
       if (cursor==markA) {
-        SetSelection(markB,model->Get().length());
+        SetSelection(markB,buffer.length());
       }
       else {
-        SetSelection(markA,model->Get().length());
+        SetSelection(markA,buffer.length());
       }
-    } else {
-      SetSelection(cursor,model->Get().length());
     }
-    cursor=model->Get().length();
+    else {
+      SetSelection(cursor,buffer.length());
+    }
+
+    cursor=buffer.length();
     Redraw();
   }
 
   bool String::ActionCursorUp()
   {
-    if (cursorUpAction.Valid()) {
-      cursorUpAction->Trigger();
-      return true;
-    }
-    else {
+    if (!cursorUpAction.Valid()) {
       return false;
     }
+
+    cursorUpAction->Trigger();
+
+    return true;
   }
 
   bool String::ActionCursorDown()
   {
-    if (cursorDownAction.Valid()) {
-      cursorDownAction->Trigger();
-      return true;
-    }
-    else {
+    if (!cursorDownAction.Valid()) {
       return false;
     }
+
+    cursorDownAction->Trigger();
+    return true;
   }
 
   void String::ActionCursorLeft()
@@ -676,7 +748,7 @@ namespace Lum {
       cursor=markB;
       ClearSelection();
     }
-    else if (cursor<model->Get().length()) {
+    else if (cursor<buffer.length()) {
       cursor++;
     }
 
@@ -690,6 +762,7 @@ namespace Lum {
     ClearSelection();
 
     cursor=0;
+
     TriggerCursorBlink();
 
     Redraw();
@@ -699,7 +772,8 @@ namespace Lum {
   {
     ClearSelection();
 
-    cursor=model->Get().length();
+    cursor=buffer.length();
+
     TriggerCursorBlink();
 
     Redraw();
@@ -707,34 +781,40 @@ namespace Lum {
 
   void String::ActionBackspace()
   {
-    if (model.Valid() && model->IsEnabled() && !readOnly) {
-      if (Selected()) {
-        cursor=markA;
-        TriggerCursorBlink();
-        DeleteSelection();
-      }
-      else if (cursor>0) {
-        ClearSelection();
-        cursor--;
-        TriggerCursorBlink();
-        model->Erase(cursor,1);
-      }
+    if (!model.Valid() ||
+        model->IsDisabled()) {
+      return;
+    }
+
+    if (Selected()) {
+      cursor=markA;
+      DeleteSelection();
+    }
+    else if (cursor>0) {
+      ClearSelection();
+      cursor--;
+      TriggerCursorBlink();
+      buffer.erase(cursor,1);
+      SyncBufferToModel();
     }
   }
 
   void String::ActionDelete()
   {
-    if (model.Valid() && model->IsEnabled() && !readOnly) {
-      if (Selected()) {
-        cursor=markA;
-        TriggerCursorBlink();
-        DeleteSelection();
-      }
-      else if (cursor<model->Get().length()) {
-        ClearSelection();
-        TriggerCursorBlink();
-        model->Erase(cursor,1);
-      }
+    if (!model.Valid() ||
+        model->IsDisabled()) {
+      return;
+    }
+
+    if (Selected()) {
+      cursor=markA;
+      DeleteSelection();
+    }
+    else if (cursor<buffer.length()) {
+      ClearSelection();
+      TriggerCursorBlink();
+      buffer.erase(cursor,1);
+      SyncBufferToModel();
     }
   }
 
@@ -773,10 +853,6 @@ namespace Lum {
 
     switch (action) {
     case actionInsert:
-      if (model->Length()+text.length()>textWidth) {
-        return false;
-      }
-
       if (Selected()) {
         DeleteSelection();
       }
@@ -784,11 +860,12 @@ namespace Lum {
       text=data.GetText();
 
       cursor+=text.length();
-      model->Insert(cursor-text.length(),text);
+      buffer.insert(cursor-text.length(),text);
+      SyncBufferToModel();
       return true;
     case actionCopy:
     case actionMove:
-      data.SetText(model->Get().substr(markA,markB-markA));
+      data.SetText(buffer.substr(markA,markB-markA));
       return true;
     default:
       return false;
@@ -797,7 +874,9 @@ namespace Lum {
 
   bool String::PostprocessAction(Base::DataExchangeObject::ExchangeAction action)
   {
-    if (!model.Valid() || action!=actionMove || !Selected()) {
+    if (!model.Valid() ||
+        action!=actionMove ||
+        !Selected()) {
       return false;
     }
 
@@ -837,10 +916,6 @@ namespace Lum {
     std::wstring help;
     int          widthA,widthB;
 
-    if (model->IsNull()) {
-      return 0;
-    }
-
     x-=textPos;
 
     if (x<=0) {
@@ -920,29 +995,38 @@ namespace Lum {
 
   void String::SelectAll()
   {
+    if (!model.Valid()) {
+      return;
+    }
+
     if (Selected()) {
       ClearSelection();
     }
 
-    if (model.Valid() && !model->IsNull()) {
-      if (model->Get().length()>0) {
-        SetSelection(0,model->Get().length());
-        cursor=0;
-      }
+    if (buffer.length()>0) {
+      SetSelection(0,buffer.length());
+      cursor=0;
     }
   }
 
   void String::DeleteSelection()
   {
+    if (!Selected()) {
+      return;
+    }
+
     int a,b;
 
-    if (Selected()) {
-      a=markA;
-      b=markB;
-      ClearSelection();
+    a=markA;
+    b=markB;
+    ClearSelection();
 
-      model->Erase(a,b-a);
-    }
+    buffer.erase(a,b-a);
+    cursor=a;
+
+    SyncBufferToModel();
+
+    TriggerCursorBlink();
   }
 
   void String::DrawCursor()
@@ -952,14 +1036,11 @@ namespace Lum {
 
   std::wstring String::GetValue() const
   {
-    if (!model.Valid() || model->IsNull()) {
-      return L"";
-    }
-    else if (mode==password) {
-      return std::wstring(model->Length(),L'*');
+    if (mode==password) {
+      return std::wstring(buffer.length(),L'*');
     }
     else {
-      return model->Get();
+      return buffer;
     }
   }
 
@@ -999,17 +1080,19 @@ namespace Lum {
     draw->disabled=(!model.Valid() || !model->IsEnabled());
   }
 
-  void String::Draw(int x, int y, size_t w, size_t h)
+  void String::Draw(OS::DrawInfo* draw,
+                    int x, int y, size_t w, size_t h)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-    bool         drawLabel=false;
+    bool drawLabel=false;
 
     draw->focused=HasFocus();
+    draw->disabled=(!model.Valid() || !model->IsEnabled());
 
-    Control::Draw(x,y,w,h);
+    Control::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       draw->focused=false;
+      draw->disabled=false;
       return;
     }
 
@@ -1033,9 +1116,7 @@ namespace Lum {
     string=GetValue();
 
     if (string.empty() &&
-        !draw->focused &&
-        model.Valid() &&
-        model->IsEnabled()) {
+        !draw->focused) {
       string=label;
       drawLabel=true;
     }
@@ -1178,9 +1259,24 @@ namespace Lum {
       //
 
       draw->selected=true;
-      back->Draw(draw,
-                 oX,oY,oWidth,oHeight,
-                 markAPos,yOff,markBPos-markAPos+1,font->height);
+      {
+        // clip on x&width: back->Draw() will not clip by itself. It will assert...
+        int markAPosClipped=markAPos;
+        size_t markWidthClipped=markBPos-markAPos+1; // BTW: why +1 ??
+        if (markAPosClipped<oX) {
+          markWidthClipped-=oX-markAPosClipped;
+          markAPosClipped=oX;
+        }
+
+        if (markAPosClipped+markWidthClipped>oX+oWidth) {
+          markWidthClipped=oX+oWidth-markAPos;
+        }
+
+        back->Draw(draw,
+                   oX,oY,oWidth,oHeight,
+                   markAPosClipped,yOff,
+                   markWidthClipped,font->height);
+      }
       draw->selected=false;
 
       draw->PushClip(markAPos,yOff,markBPos-markAPos+1,height);
@@ -1199,6 +1295,7 @@ namespace Lum {
       draw->DrawString(textPos,yOff+font->ascent,string);
       draw->PopForeground();
     }
+
     draw->PopFont();
 
     /* Drawing the cursor */
@@ -1207,6 +1304,7 @@ namespace Lum {
         draw->PushForeground(OS::Display::fillColor);
         draw->FillRectangle(cursorPos,yOff,1,font->height);
         draw->PopForeground();
+
         draw->PushFont(font);
         draw->PushForeground(OS::Display::fillTextColor);
         draw->DrawString(cursorPos,yOff,string.substr(cursor,1));
@@ -1238,15 +1336,12 @@ namespace Lum {
     draw->PopClip();
 
     draw->focused=false;
+    draw->disabled=false;
   }
 
   void String::SetCursorPos(size_t pos)
   {
-    std::wstring string;
-
-    string=model->Get();
-
-    assert(pos<=string.length());
+    assert(pos<=buffer.length());
 
     cursor=pos;
 
@@ -1283,7 +1378,8 @@ namespace Lum {
 
   bool String::HandleMouseEvent(const OS::MouseEvent& event)
   {
-    if (!visible || !model.Valid() || model->IsNull()) {
+    if (!visible ||
+        !model.Valid()) {
       return false;
     }
 
@@ -1340,8 +1436,6 @@ namespace Lum {
           SetSelection(help,cursor);
           Redraw();
         }
-
-        OpenInputHelper();
       }
     }
     else if (event.type==OS::MouseEvent::move &&
@@ -1367,10 +1461,6 @@ namespace Lum {
 
   bool String::HandleKeys(const OS::KeyEvent& event)
   {
-    if (!model.Valid() || !model->IsEnabled() || model->IsNull()) {
-      return false;
-    }
-
     switch (event.key) {
     case OS::keyTab:
       return false;
@@ -1385,19 +1475,17 @@ namespace Lum {
     case OS::keyEscape:
       break;
     default:
-      if (!readOnly) {
-        unsigned long qualifier;
-        std::wstring  name;
+      unsigned long qualifier;
+      std::wstring  name;
 
-        event.GetName(name);
-        qualifier=event.qualifier & ~(OS::qualifierShiftLeft|OS::qualifierShiftRight|OS::qualifierShift|OS::qualifierCapsLock);
+      event.GetName(name);
+      qualifier=event.qualifier & ~(OS::qualifierShiftLeft|OS::qualifierShiftRight|OS::qualifierShift|OS::qualifierCapsLock);
 
-        if (!event.text.empty() &&
-            !(name==event.text && qualifier!=0) &&
-            event.text[0]>=32) {
-          ActionInsertString(event.text);
-          return true;
-        }
+      if (!event.text.empty() &&
+          !(name==event.text && qualifier!=0) &&
+          event.text[0]>=32) {
+        ActionInsertString(event.text);
+        return true;
       }
     }
 
@@ -1407,9 +1495,8 @@ namespace Lum {
   void String::EvaluateMarkAllActionRights()
   {
     if (!this->model.Valid() ||
-        this->model->IsNull() ||
-        this->model->Get().length()==0 ||
-        (markA==0 && markB==this->model->Get().length())) {
+        buffer.length()==0 ||
+        (markA==0 && markB==buffer.length())) {
       markAllAction->Disable();
     }
     else {
@@ -1501,14 +1588,18 @@ namespace Lum {
       ActionSelectAll();
     }
     else if (model==this->model) {
-      if (Selected()) {
-        SelectAll();
-      }
+      if (sync) {
+         SyncModelToBuffer();
 
-      EvaluateMarkAllActionRights();
+        if (Selected()) {
+          SelectAll();
+        }
 
-      if (visible) {
-        Redraw();
+        EvaluateMarkAllActionRights();
+
+        if (visible) {
+          Redraw();
+        }
       }
     }
 
@@ -1548,210 +1639,5 @@ namespace Lum {
 
     return s;
   }
-
-  String::InputHelper::InputHelper()
-   : backAction(new Model::Action()),
-     string(NULL)
-  {
-    Observe(backAction);
-  }
-
-  void String::InputHelper::RegisterKey(Model::Action* action, const std::wstring& string)
-  {
-    assert(action!=NULL);
-    assert(!string.empty());
-
-    Observe(action);
-
-    strings[action]=string;
-  }
-
-  Button* String::InputHelper::CreateTextButton(const std::wstring& label,
-                                                const std::wstring& text)
-  {
-    Button           *button;
-    Model::ActionRef action=new Model::Action();
-
-    button=Button::Create(label,action,true,true);
-
-    RegisterKey(action,text);
-
-    return button;
-  }
-
-  Button* String::InputHelper::CreateTextButton(const std::wstring& label,
-                                                Model::Action* action)
-  {
-    return Button::Create(label,action,true,true);
-  }
-
-  bool String::InputHelper::Open(String *string)
-  {
-    this->string=string;
-
-    return Popup::Open(false);
-  }
-
-  void String::InputHelper::Close()
-  {
-    Popup::Close();
-  }
-
-  void String::InputHelper::Resync(Base::Model* model, const Base::ResyncMsg& msg)
-  {
-    if (string!=NULL) {
-      if (model==backAction &&
-          backAction->IsFinished()) {
-        string->ActionBackspace();
-      }
-      else if (dynamic_cast<Model::Action*>(model)!=NULL &&
-          dynamic_cast<Model::Action*>(model)->IsFinished()) {
-        std::map<Model::ActionRef,std::wstring>::iterator iter=strings.find(dynamic_cast<Model::Action*>(model));
-
-        if (iter!=strings.end()) {
-          string->ActionInsertString(iter->second);
-        }
-      }
-    }
-
-    Popup::Resync(model,msg);
-  }
-
-  InputHelperNumber::InputHelperNumber(Type type)
-   : type(type)
-  {
-    // no code
-  }
-
-  void InputHelperNumber::PreInit()
-  {
-    Array *array=new Array();
-
-    array->SetFlex(false,false);
-    array->SetSpace(false,false);
-
-    if (type==typeBin) {
-      array->SetHorizontalCount(3);
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typeOct) {
-      array->SetHorizontalCount(5);
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->AddEmpty();
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typePosDec) {
-      array->SetHorizontalCount(3);
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(CreateTextButton(L"8",L"8"));
-      array->Add(CreateTextButton(L"9",L"9"));
-
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->AddEmpty();
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typeDec) {
-      array->SetHorizontalCount(3);
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(CreateTextButton(L"8",L"8"));
-      array->Add(CreateTextButton(L"9",L"9"));
-
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(L"-",L"-"));
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typeHex) {
-      array->SetHorizontalCount(9);
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(CreateTextButton(L"8",L"8"));
-      array->Add(CreateTextButton(L"9",L"9"));
-      array->Add(CreateTextButton(L"A",L"A"));
-      array->Add(CreateTextButton(L"B",L"B"));
-      array->Add(CreateTextButton(L"C",L"C"));
-      array->Add(CreateTextButton(L"D",L"D"));
-      array->Add(CreateTextButton(L"E",L"E"));
-      array->Add(CreateTextButton(L"F",L"F"));
-      array->AddEmpty();
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typePosFloat) {
-      array->SetHorizontalCount(3);
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(CreateTextButton(L"8",L"8"));
-      array->Add(CreateTextButton(L"9",L"9"));
-
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(Lum::Base::GetDecimalPointWString(),
-                                  Lum::Base::GetDecimalPointWString()));
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-    else if (type==typeFloat) {
-      array->SetHorizontalCount(4);
-      array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(CreateTextButton(L"8",L"8"));
-      array->Add(CreateTextButton(L"9",L"9"));
-      array->Add(CreateTextButton(L"-",L"-"));
-
-      array->Add(CreateTextButton(L"4",L"4"));
-      array->Add(CreateTextButton(L"5",L"5"));
-      array->Add(CreateTextButton(L"6",L"6"));
-      array->Add(CreateTextButton(L"+",L"+"));
-
-      array->Add(CreateTextButton(L"1",L"1"));
-      array->Add(CreateTextButton(L"2",L"2"));
-      array->Add(CreateTextButton(L"3",L"3"));
-      array->Add(CreateTextButton(L"e",L"e"));
-
-      array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(CreateTextButton(Lum::Base::GetDecimalPointWString(),
-                                  Lum::Base::GetDecimalPointWString()));
-      array->AddEmpty();
-      array->Add(CreateTextButton(L"<-",backAction));
-    }
-
-    SetMain(array);
-
-    Popup::PreInit();
-  }
 }
 
