@@ -19,7 +19,7 @@
 
 #include <Lum/Table.h>
 
-#include <Lum/OS/Behaviour.h>
+#include <Lum/Manager/Behaviour.h>
 
 #include <Lum/Panel.h>
 #include <Lum/PosMarker.h>
@@ -33,14 +33,16 @@ namespace Lum {
     kineticScroller(new KineticScroller()),
     hVisible(false),vVisible(false),
     table(new TableView()),
-    header(NULL),headerModel(new Model::HeaderImpl()),showHeader(false),
-    refreshAction(new Model::Action())
+    header(new Header()),headerModel(new Model::HeaderImpl()),showHeader(false)
   {
     SetBackground(OS::display->GetFill(OS::Display::scrolledBackgroundFillIndex));
 
     headerModel->AddColumn(L"",Base::Size::pixel,32000,true);
 
-    Observe(refreshAction);
+    Observe(table->GetHAdjustment()->GetTopModel());
+    Observe(table->GetVAdjustment()->GetTopModel());
+    Observe(table->GetHAdjustment()->GetTotalModel());
+    Observe(table->GetVAdjustment()->GetTotalModel());
   }
 
   Table::~Table()
@@ -113,6 +115,26 @@ namespace Lum {
     showHeader=show;
   }
 
+  void Table::SetAutoFitColumns(bool autoFit)
+  {
+    table->SetAutoFitColumns(autoFit);
+  }
+
+  void Table::SetAutoHSize(bool autoSize)
+  {
+    table->SetAutoHSize(autoSize);
+  }
+
+  void Table::SetAutoVSize(bool autoSize)
+  {
+    table->SetAutoVSize(autoSize);
+  }
+
+  void Table::SetRowHeight(size_t height)
+  {
+    table->SetRowHeight(height);
+  }
+
   bool Table::SetModel(Base::Model* model)
   {
     return table->SetModel(model);
@@ -128,28 +150,28 @@ namespace Lum {
     table->SetDoubleClickAction(action);
   }
 
-  void Table::SetFormatProvider(TableView::FormatProvider* formatProvider)
+  void Table::SetMouseSelectionAction(Model::Action* action)
   {
-    table->SetFormatProvider(formatProvider);
+    table->SetMouseSelectionAction(action);
+  }
+
+  void Table::SetDefaultAction(Model::Action* action)
+  {
+    table->SetDefaultAction(action);
+  }
+
+  void Table::SetPainter(TableCellPainter* painter)
+  {
+    table->SetPainter(painter);
   }
 
   void Table::CalcSize()
   {
-    if (table==NULL) {
-      table=new TableView();
-    }
-
     if (!inited) {
       table->SetFlex(true,true);
       table->SetHeader(headerModel);
+      table->SetScrollView(this);
       kineticScroller->SetObjects(this,table);
-
-      Observe(table->GetHAdjustment());
-      Observe(table->GetVAdjustment());
-    }
-
-    if (header==NULL) {
-      header=new Header();
     }
 
     if (!inited) {
@@ -173,7 +195,7 @@ namespace Lum {
 
 
     if (hScroller==NULL) {
-      if (!OS::display->GetBehaviour()->ScrollKnobActive() &&
+      if (!Manager::Behaviour::Instance()->ScrollKnobActive() &&
           !table->RequiresKnobs()) {
         hScroller=new PosMarker(false);
       }
@@ -187,7 +209,7 @@ namespace Lum {
     }
 
     if (vScroller==NULL) {
-      if (!OS::display->GetBehaviour()->ScrollKnobActive() &&
+      if (!Manager::Behaviour::Instance()->ScrollKnobActive() &&
           !table->RequiresKnobs()) {
         vScroller=new PosMarker(true);
       }
@@ -230,37 +252,56 @@ namespace Lum {
 
   void Table::Layout()
   {
-    hScroller->ResizeWidth(GetOWidth());
-    hScroller->Layout();
-    vScroller->ResizeHeight(GetOHeight());
-    vScroller->Layout();
-
-    if (hVisible &&
-        (!table->GetHAdjustment()->IsValid() ||
-         table->GetHAdjustment()->IsCompletelyVisible())) {
-      hScroller->Hide();
-      hVisible=false;
-    }
-    else if (!hVisible &&
-             table->GetHAdjustment()->IsValid() &&
-             !table->GetHAdjustment()->IsCompletelyVisible()) {
-      hVisible=true;
-    }
-
-    if (vVisible &&
-        (!table->GetVAdjustment()->IsValid() ||
-         table->GetVAdjustment()->IsCompletelyVisible())) {
-      vScroller->Hide();
-      vVisible=false;
-    }
-    else if (!vVisible &&
-             table->GetVAdjustment()->IsValid() &&
-             !table->GetVAdjustment()->IsCompletelyVisible()) {
-      vVisible=true;
-    }
-
     size_t tWidth,tHeight;
 
+    hScroller->ResizeWidth(GetOWidth());
+    vScroller->ResizeHeight(GetOHeight());
+
+    tWidth=width;
+    tHeight=height;
+    if (object->GetBackground()!=NULL) {
+      tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+      tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+    }
+    table->GetODimension(tWidth,tHeight);
+    if (object->GetBackground()!=NULL) {
+      tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+      tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+    }
+
+    hVisible=width<tWidth;
+    vVisible=height<tHeight;
+
+    // Showing one scrollbar reduces the visible table bar, so the other scrollbar
+    // might be shown, too
+
+    if (hVisible && !vVisible) {
+      tHeight=height-hScroller->GetOHeight();
+      if (object->GetBackground()!=NULL) {
+        tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      table->GetODimension(tWidth,tHeight);
+      if (object->GetBackground()!=NULL) {
+        tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      vVisible=height-hScroller->GetOHeight()<tHeight;
+    }
+    else if (!hVisible && vVisible) {
+      tWidth=width-vScroller->GetOWidth();
+      if (object->GetBackground()!=NULL) {
+        tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      table->GetODimension(tWidth,tHeight);
+      if (object->GetBackground()!=NULL) {
+        tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      hVisible=width-vScroller->GetOWidth()<tWidth;
+    }
+
     tWidth=width;
     tHeight=height;
 
@@ -274,21 +315,21 @@ namespace Lum {
       tWidth-=vScroller->GetOWidth();
     }
 
-    // Resize and position objects
-
-    if (tWidth!=object->GetOWidth() ||
-        tHeight!=object->GetOHeight())  {
-      object->MoveResize(x,y,tWidth,tHeight);
-      object->Layout();
-    }
-
     if (hVisible) {
       hScroller->MoveResize(x,y+tHeight,tWidth,hScroller->GetOHeight());
     }
+    else {
+      hScroller->Hide();
+    }
 
     if (vVisible) {
       vScroller->MoveResize(x+tWidth,y,vScroller->GetOWidth(),tHeight);
     }
+    else {
+      vScroller->Hide();
+    }
+
+    object->MoveResize(x,y,tWidth,tHeight);
 
     Group::Layout();
   }
@@ -328,33 +369,9 @@ namespace Lum {
       return;
     }
 
-    if (model==table->GetHAdjustment() ||
-        model==table->GetVAdjustment()) {
-      if (visible) {
-        OS::display->QueueActionForEventLoop(refreshAction);
-      }
-      else {
-        SetRelayout();
-      }
-      return;
-    }
-    else if (model==refreshAction &&
-             refreshAction->IsFinished()) {
-      if ((hVisible &&
-           (!table->GetHAdjustment()->IsValid() ||
-            table->GetHAdjustment()->IsCompletelyVisible())) ||
-          (!hVisible &&
-           table->GetHAdjustment()->IsValid() &&
-           !table->GetHAdjustment()->IsCompletelyVisible()) ||
-          (vVisible &&
-           (!table->GetVAdjustment()->IsValid() ||
-            table->GetVAdjustment()->IsCompletelyVisible())) ||
-          (!vVisible &&
-           table->GetVAdjustment()->IsValid() &&
-           !table->GetVAdjustment()->IsCompletelyVisible())) {
-        SetRelayout();
-        Redraw();
-      }
+    if (model==table->GetHAdjustment()->GetTotalModel() ||
+        model==table->GetVAdjustment()->GetTotalModel()) {
+      SetRelayout();
     }
 
     if (!table->RequiresKnobs()) {
