@@ -25,48 +25,24 @@
 
 #include <Lum/Base/Size.h>
 
+#include <Lum/Manager/Behaviour.h>
+
 #include <Lum/Table.h>
 
 namespace Lum {
 
-  static ControllerRef controller(new TableView::Controller);
-
-  TableView::FormatProvider::~FormatProvider()
-  {
-    // no code
-  }
-
-  TableView::Alignment TableView::FormatProvider::GetAlignment(size_t /*column*/, size_t /*row*/) const
-  {
-    return left;
-  }
-
-  OS::Color TableView::FormatProvider::GetTextColor(size_t column, size_t row) const
-  {
-    return OS::display->GetColor(OS::Display::tableTextColor);
-  }
-
-  OS::Fill* TableView::FormatProvider::GetBackground(size_t column, size_t row) const
-  {
-    if (row%2!=0) {
-      return OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
-    }
-    else {
-      return OS::display->GetFill(OS::Display::tableBackground2FillIndex);
-    }
-  }
+  static ControllerRef controller(new TableView::Controller());
 
   TableView::TableView()
-  : doubleClickAction(new Model::Action),
-    mouseSelectionAction(new Model::Action),
-    autoFitColumns(false),
+  : autoFitColumns(false),
     autoHSize(false),
     autoVSize(false),
     tableWidth(0),
     rowHeight(0),
+    refreshes(0),
     fullRefresh(true),
     top(0),
-    formatProvider(new FormatProvider()),
+    painter(NULL),
     font(OS::display->GetFont())
   {
     SetClipFromParent(true);
@@ -77,11 +53,15 @@ namespace Lum {
     RequestFocus();
 
     SetController(Lum::controller);
+
+    SetDoubleClickAction(new Model::Action());
+    SetMouseSelectionAction(new Model::Action());
+    SetDefaultAction(new Model::Action());
   }
 
   TableView::~TableView()
   {
-    delete formatProvider;
+    // no code
   }
 
   bool TableView::RequiresKnobs() const
@@ -210,6 +190,18 @@ namespace Lum {
 
     if (this->model.Valid())  {
       vAdjustment->SetTop(1);
+
+      if (painter==NULL) {
+        if (dynamic_cast<Model::StringTable*>(model)!=NULL) {
+          SetPainter(new StringTablePainter());
+        }
+        else if (dynamic_cast<Model::StringRefTable*>(model)!=NULL) {
+          SetPainter(new StringRefTablePainter());
+        }
+        else if (dynamic_cast<Model::StringMatrix*>(model)!=NULL) {
+          SetPainter(new StringMatrixPainter());
+        }
+      }
     }
 
     Control::SetModel(this->model);
@@ -259,23 +251,41 @@ namespace Lum {
 
   void TableView::SetDoubleClickAction(Model::Action* action)
   {
+    if (doubleClickAction.Valid()) {
+      Forget(doubleClickAction);
+    }
+
     doubleClickAction=action;
+
+    if (doubleClickAction.Valid()) {
+      Observe(doubleClickAction);
+    }
+  }
+
+  void TableView::SetDefaultAction(Model::Action* action)
+  {
+    if (defaultAction.Valid()) {
+      Forget(defaultAction);
+    }
+
+    defaultAction=action;
+
+    if (defaultAction.Valid()) {
+      Observe(defaultAction);
+    }
   }
 
   /**
-    Assign a new FormatProvider.
-
-    Previously assigned provider will be deleted.
-    The resource ownership of the assigned format provider will be hold by the table.
-    You must not free an assigned format provider anytime after the assignment.
-  */
-  void TableView::SetFormatProvider(FormatProvider* formatProvider)
+    Assign a table painter to the table. A table painter is responsible for
+    doing the real row and cell rendering.
+    */
+  void TableView::SetPainter(TableCellPainter* painter)
   {
-    assert(formatProvider!=NULL);
-
-    delete this->formatProvider;
-    this->formatProvider=formatProvider;
+    this->painter=painter;
+    this->painter->SetParent(this);
+    this->painter->SetModel(model);
 
+    refreshes++;
     Redraw();
   }
 
@@ -299,61 +309,60 @@ namespace Lum {
     return mouseSelectionAction.Get();
   }
 
+  Model::Action* TableView::GetDefaultAction() const
+  {
+    return defaultAction.Get();
+  }
+
   void TableView::FitColumn(size_t column)
   {
     size_t width;
 
-    if (!model.Valid() || !header.Valid()) {
+    if (!model.Valid() || !header.Valid() || painter==NULL) {
       return;
     }
 
     width=2*OS::display->GetSpaceHorizontal(OS::Display::spaceObjectBorder);
-    for (size_t y=1; y<=model->GetRows(); y++) {
-      std::wstring string=model->GetString(column,y);
-      if (!string.empty()) {
-        OS::FontExtent extent;
+    for (size_t i=1; i<model->GetRows(); i++) {
+      painter->SetCell(column,i);
+      width=std::max(width,painter->GetProposedWidth());
+    }
+    header->SetColumnWidth(column-1,width);
+  }
 
-        font->StringExtent(string,extent);
-        width=std::max(width,extent.width);
+  void TableView::ResyncWithModel()
+  {
+    CalculateRowHeight();
+
+    if (this->model.Valid() &&
+        !this->model->IsNull() &&
+        this->model->GetRows()>0) {
+      if (rowHeight!=0) {
+        vAdjustment->SetStepSize(rowHeight);
+        vAdjustment->Set(1,height,this->model->GetRows()*rowHeight);
       }
       else {
-        Object *object=model->GetObject(column,y);
-        if (object!=NULL)  {
-          if (!inited || (object->GetWindow()!=GetWindow()))  {
-            object->SetWindow(GetWindow());
-            object->SetParent(this);
-            object->CalcSize();
-          }
-          width=std::max(width,(size_t)object->GetOWidth());
-        }
+        vAdjustment->SetInvalid();
       }
     }
-
-    header->SetColumnWidth(column-1,width);
+    else {
+      vAdjustment->SetInvalid();
+    }
   }
 
   void TableView::CalculateRowHeight()
   {
-    if (rowHeight!=0 || !model.Valid() || model->GetRows()==0) {
+    if (rowHeight!=0 || !model.Valid() || model->GetRows()==0 || !header.Valid()) {
       return;
     }
 
     size_t rh=0;
 
-    for (size_t x=1; x<=this->header->GetColumns(); x++) {
-      Object *object=this->model->GetObject(x,1);
-
-      if (object!=NULL) {
-        if (!object->IsInited()) {
-          object->SetParent(this);
-          object->SetWindow(GetWindow());
-          object->CalcSize();
-        }
-
-        rh=std::max(rh,object->GetOHeight());
-
-      }
-      else {
+    if (painter.Valid()) {
+      rh=painter->GetProposedHeight();
+    }
+    else {
+      for (size_t x=1; x<=this->header->GetColumns(); x++) {
         rh=std::max(rh,font->height);
       }
     }
@@ -361,50 +370,45 @@ namespace Lum {
     SetRowHeight(rh);
   }
 
-  void TableView::FitColumns()
+  void TableView::FitColumns(size_t targetWidth)
   {
-    size_t targetWidth=this->width;
-
-    // This is a hack!
-    // TableView does know, that GetParent()->GetParent() is normally of type
-    // Table and requests width of vertical scroller to resize labels that way
-    // that content does not scroll out of view...
-    if (header->GetColumns()>1 &&
-        firstDraw &&
-        GetParent()!=NULL && dynamic_cast<Table*>(GetParent()->GetParent())!=NULL/* &&
-        vAdjustment->GetVisible()<hAdjustment->GetTotal()*/) {
-      targetWidth-=dynamic_cast<Table*>(GetParent()->GetParent())->GetVerticalScrollerWidth();
-    }
-
     if (header->GetColumns()==1) {
       header->SetColumnWidth(0,targetWidth);
     }
     else {
-      size_t w=0;
+      size_t fitWidth=0;
       size_t flexs=0;
 
       for (size_t column=0; column<header->GetColumns(); column++) {
-        w+=header->GetColumnWidth(column);
+        fitWidth+=header->GetColumnWidth(column);
         if (header->CanColumnChangeSize(column)) {
           ++flexs;
         }
       }
 
-      if (w!=targetWidth && flexs>0) {
+      if (fitWidth!=targetWidth && flexs>0) {
         for (size_t column=0; column<header->GetColumns(); column++) {
           if (header->CanColumnChangeSize(column)) {
-            if (w<targetWidth) {
-              header->SetColumnWidth(column,header->GetColumnWidth(column)+(targetWidth-w)/flexs);
-              w+=(targetWidth-w)/flexs;
+            if (fitWidth<targetWidth) {
+              header->SetColumnWidth(column,
+                                     header->GetColumnWidth(column)+
+                                     (targetWidth-fitWidth)/flexs);
+              fitWidth+=(targetWidth-fitWidth)/flexs;
             }
             else {
-              header->SetColumnWidth(column,header->GetColumnWidth(column)-(w-targetWidth)/flexs);
-              w+=(w-targetWidth)/flexs;
+              header->SetColumnWidth(column,
+                                     header->GetColumnWidth(column)-
+                                     (fitWidth-targetWidth)/flexs);
+              fitWidth+=(fitWidth-targetWidth)/flexs;
             }
           }
         }
 
-        RecalcTableWidth();
+        size_t newWidth=0;
+
+        for (size_t column=0; column<header->GetColumns(); column++) {
+          newWidth+=header->GetColumnWidth(column);
+        }
       }
     }
   }
@@ -450,35 +454,9 @@ namespace Lum {
     return true;
   }
 
-  void TableView::UpdateDimensions()
-  {
-    if (model.Valid() &&
-        !model->IsNull() &&
-        (model->GetRows()>0 ||
-         (header.Valid() && header->GetColumns()>1))) {
-      hAdjustment->SetStepSize(font->StringWidth(L"m"));
-      hAdjustment->SetDimension(width,tableWidth);
-    }
-    else {
-      hAdjustment->SetInvalid();
-    }
-
-    if (model.Valid() &&
-        !model->IsNull() &&
-        model->GetRows()>0) {
-      if (rowHeight!=0) {
-        vAdjustment->SetStepSize(rowHeight);
-        vAdjustment->SetDimension(height,model->GetRows()*rowHeight);
-      }
-    }
-    else {
-      vAdjustment->SetInvalid();
-    }
-  }
-
   /* ---------- Printing --------------- */
 
-  OS::Color TableView::GetTextColor(OS::DrawInfo *draw) const
+  OS::Color TableView::GetTextColor(OS::DrawInfo *draw, const void* child) const
   {
     if (draw->selected) {
       return OS::display->GetColor(OS::Display::fillTextColor);
@@ -511,116 +489,44 @@ namespace Lum {
                            int y,
                            bool drawBackground)
   {
-    Alignment    alignment;
-    bool         selected,cliped;
-    std::wstring string;
-    Object       *object;
-    OS::Fill*    back;
-
-    selected=selection.Valid() && (selection->IsLineSelected(y) || selection->IsCellSelected(x,y));
-    alignment=formatProvider->GetAlignment(x,y);
+    if (painter==NULL) {
+      return;
+    }
 
+    OS::FillRef back;
+    bool        selected=selection.Valid() &&
+                         (selection->IsLineSelected(y) || selection->IsCellSelected(x,y));
     if (selected) {
-      back=OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
       draw->selected=true;
     }
-    else {
-      back=formatProvider->GetBackground(x,y);
-    }
-
-    cliped=false;
-    string=model->GetString(x,y);
-    object=model->GetObject(x,y);
-
-    if (!string.empty()) {
-      OS::FontExtent extent;
-      int            textStart=xPos; // To make compiler happy
-
-      if (drawBackground) {
-        back->Draw(draw,xPos,yPos,width,height,xPos,yPos,width,height);
-      }
 
+    if (drawBackground) {
       if (selected) {
-        draw->PushForeground(OS::Display::fillTextColor);
+        back=OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
       }
       else {
-        draw->PushForeground(formatProvider->GetTextColor(x,y));
+        painter->SetCell(x,y);
+        back=painter->GetCellBackground(draw);
+
+        if (!back.Valid()) {
+          if (y%2!=0) {
+            back=OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
+          }
+          else {
+            back=OS::display->GetFill(OS::Display::tableBackground2FillIndex);
+          }
+        }
       }
 
-      font->StringExtent(string,extent);
-      cliped=extent.width>width;
-
-      if (cliped) {
-        draw->PushClip(xPos,yPos,width,height);
-      }
-
-      switch (alignment) {
-      case center:
-        textStart=xPos+(width-extent.width) / 2;
-        break;
-      case right:
-        textStart=xPos+width-extent.width;
-        break;
-      case left:
-        textStart=xPos;
-        break;
-      }
-
-      draw->DrawString(textStart,yPos+font->ascent+((int)height-(int)font->height) / 2,string);
-
-      draw->PopForeground();
-    }
-    else if (object!=NULL) {
-      bool focused=draw->focused;
-
-      draw->focused=false;
-
-      if (!inited || (object->GetWindow()!=GetWindow())) {
-        object->SetWindow(GetWindow());
-        object->SetParent(this);
-        object->CalcSize();
-      }
-
-      object->Resize(width,height);
-      object->Move(xPos,yPos+((int)height-(int)object->GetOHeight()) / 2);
-      object->Layout();
-
-      if (drawBackground) {
-        back->Draw(draw,xPos,yPos,width,height,xPos,yPos,width,height);
-      }
-
-      cliped=object->GetOWidth()>width;
-
-      if (cliped) {
-        draw->PushClip(xPos,yPos,width,height);
-      }
-
-      if (object->GetBackground()==NULL) {
-        object->SetBackground(new Lum::OS::EmptyFill());
-      }
-      object->Draw(xPos,yPos,object->GetOWidth(),object->GetOHeight());
-
-      draw->focused=focused;
-    }
-    else if (drawBackground) {
       back->Draw(draw,xPos,yPos,width,height,xPos,yPos,width,height);
     }
 
-    if (selected &&
-        selection.Valid() &&
-        !selection->IsLineSelected(y) &&
-        HasFocus()) {
-      OS::FrameRef focus=OS::display->GetFrame(OS::Display::focusFrameIndex);
+    painter->SetCell(x,y);
+    painter->Draw(draw,xPos,yPos,width,height);
 
-      /* Draw a frame arond one cell */
-      focus->Draw(draw,xPos,yPos,width,height);
+    if (selected) {
+      draw->selected=false;
     }
-
-    if (cliped) {
-      draw->PopClip();
-    }
-
-    draw->selected=false;
   }
 
   /**
@@ -731,6 +637,7 @@ namespace Lum {
     size_t lh=0;
 
     if (GetLineBounds(line,ly,lh)) {
+      refreshes++;
       Redraw(this->x,ly,width,lh);
     }
   }
@@ -745,40 +652,27 @@ namespace Lum {
     if (autoVSize && model.Valid()) {
       height=model->GetRows()*std::max(font->height,OS::display->GetTheme()->GetMinimalTableRowHeight());
       if (header->GetColumns()==1 && !autoHSize) {
-        // This is a hack!
-        // TableView does know, that GetParent()->GetParent() is normally of type
-        // Table and requests width of vertical scroller to resize width that all labels
-        // fit.
-        if (GetParent()!=NULL && dynamic_cast<Table*>(GetParent()->GetParent())!=NULL) {
-          height+=dynamic_cast<Table*>(GetParent()->GetParent())->GetHorizontalScrollerHeight();
-        }
       }
     }
     else {
       height=minHeight;
     }
 
-    if (autoHSize && model.Valid() && header.Valid()) {
-      size_t w=0;
+    if (autoHSize &&
+        model.Valid() &&
+        header.Valid()) {
+      size_t autoWidth=0;
 
       for (size_t column=0; column<header->GetColumns(); column++) {
         if (header->GetMinColumnWidth(column)==0) {
           FitColumn(column+1);
         }
-        w+=header->GetColumnWidth(column);
+        autoWidth+=header->GetColumnWidth(column);
       }
 
-      // This is a hack!
-      // TableView does know, that GetParent()->GetParent() is normally of type
-      // Table and requests width of vertical scroller to resize width that all labels
-      // fit.
-      if (GetParent()!=NULL && dynamic_cast<Table*>(GetParent()->GetParent())!=NULL) {
-        w+=dynamic_cast<Table*>(GetParent()->GetParent())->GetVerticalScrollerWidth();
-      }
-
-      width=w;
-      minWidth=w;
-      tableWidth=w;
+      width=autoWidth;
+      minWidth=autoWidth;
+      tableWidth=autoWidth;
     }
     else {
       RecalcTableWidth();
@@ -787,20 +681,58 @@ namespace Lum {
     Scrollable::CalcSize();
   }
 
-  void TableView::Layout()
+  void TableView::GetDimension(size_t& width, size_t& height)
   {
     CalculateRowHeight();
 
-    if (model.Valid() && header.Valid() && autoFitColumns) {
-      FitColumns();
+    if (model.Valid() &&
+        !model->IsNull() &&
+        (model->GetRows()>0 ||
+         (header.Valid()))) {
+      if (autoFitColumns) {
+        FitColumns(width);
+      }
+      RecalcTableWidth();
+      width=tableWidth;
+    }
+
+    if (model.Valid() &&
+        !model->IsNull() &&
+        model->GetRows()>0) {
+      if (rowHeight!=0) {
+        height=model->GetRows()*rowHeight;
+      }
+    }
+  }
+
+  void TableView::Layout()
+  {
+    ResyncWithModel();
+
+    if (model.Valid() &&
+        !model->IsNull() &&
+        (model->GetRows()>0 ||
+         (header.Valid()))) {
+      if (autoFitColumns) {
+        FitColumns(width);
+      }
+
+      RecalcTableWidth();
+
+      hAdjustment->SetStepSize(font->StringWidth(L"m"));
+      hAdjustment->SetDimension(width,tableWidth);
+    }
+    else {
+      hAdjustment->SetInvalid();
     }
 
     Scrollable::Layout();
   }
 
-  void TableView::Draw(int x, int y, size_t w, size_t h)
+  void TableView::Draw(OS::DrawInfo* draw,
+                       int x, int y, size_t w, size_t h)
   {
-    Scrollable::Draw(x,y,w,h);
+    Scrollable::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
@@ -808,9 +740,10 @@ namespace Lum {
 
     /* --- */
 
-    CalculateRowHeight();
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
+    if (refreshes>1) {
+      fullRefresh=true;
+    }
+    refreshes=0;
 
     if (!model.Valid() || model->GetRows()==0) {
       std::wstring emptyText;
@@ -837,8 +770,6 @@ namespace Lum {
       return;
     }
 
-    RecalcTableWidth();
-
     if (OS::display->GetType()==OS::Display::typeTextual) {
       DrawFull(draw);
       return;
@@ -1177,6 +1108,7 @@ namespace Lum {
     const Model::Header::FitColumn      *fitColumn;
     const Model::Selection::RefreshCell *selRefreshCell;
     const Model::Selection::RefreshRow  *selRefreshRow;
+    const Model::Selection::Selected    *selected;
 
     if ((refreshCell=dynamic_cast<const Model::Table::RefreshCell*>(&msg))!=NULL) {
       RedrawLine(refreshCell->row);
@@ -1185,7 +1117,11 @@ namespace Lum {
       RedrawLine(refreshRow->row);
     }
     else if ((insertRow=dynamic_cast<const Model::Table::InsertRow*>(&msg))!=NULL) {
+      CalculateRowHeight();
+      vAdjustment->SetDimension(height,this->model->GetRows()*rowHeight);
+
       fullRefresh=true;
+      refreshes++;
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
     else if ((deleteRow=dynamic_cast<const Model::Table::DeleteRow*>(&msg))!=NULL) {
@@ -1194,20 +1130,44 @@ namespace Lum {
           selection->OnLineDeleted(r);
         }
       }
+      CalculateRowHeight();
+      vAdjustment->SetDimension(height,this->model->GetRows()*rowHeight);
       fullRefresh=true;
+      refreshes++;
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
     else if ((resizedColumn=dynamic_cast<const Model::Header::ResizedColumn*>(&msg))!=NULL) {
+      refreshes++;
       Redraw(); // TODO: Calculate rectangle to get redrawn...
     }
     else if ((fitColumn=dynamic_cast<const Model::Header::FitColumn*>(&msg))!=NULL) {
       FitColumn(fitColumn->column+1);
     }
-    else if ((selRefreshCell=dynamic_cast<const Model::Selection::RefreshCell*>(&msg))!=NULL) {
-      RedrawLine(selRefreshCell->row);
+    else if (model==selection) {
+      if ((selRefreshCell=dynamic_cast<const Model::Selection::RefreshCell*>(&msg))!=NULL) {
+        RedrawLine(selRefreshCell->row);
+        }
+      else if ((selRefreshRow=dynamic_cast<const Model::Selection::RefreshRow*>(&msg))!=NULL) {
+        RedrawLine(selRefreshRow->row);
+      }
+      else if ((selected=dynamic_cast<const Model::Selection::Selected*>(&msg))!=NULL &&
+               defaultAction.Valid() &&
+               defaultAction->IsEnabled() &&
+               selection->HasSelection() &&
+               Manager::Behaviour::Instance()->GetTableDefaultActionMode()==Manager::Behaviour::TableActionOnSelection) {
+        defaultAction->Trigger();
+      }
     }
-    else if ((selRefreshRow=dynamic_cast<const Model::Selection::RefreshRow*>(&msg))!=NULL) {
-      RedrawLine(selRefreshRow->row);
+    else if (model==doubleClickAction) {
+      if (doubleClickAction->IsFinished() &&
+          doubleClickAction->IsEnabled() &&
+          defaultAction.Valid() &&
+          defaultAction->IsEnabled() &&
+          selection.Valid() &&
+          selection->HasSelection() &&
+          Manager::Behaviour::Instance()->GetTableDefaultActionMode()==Manager::Behaviour::TableActionOnDoubleclick) {
+        defaultAction->Trigger();
+      }
     }
     else { // unknown/empty message
       if (model==this->model) {
@@ -1216,22 +1176,19 @@ namespace Lum {
           selection->OnClear();
         }
 
-        fullRefresh=true;
-        UpdateDimensions();
+        ResyncWithModel();
 
-        if (this->model->GetRows()>0) {
-          vAdjustment->SetTop(1);
-        }
+        fullRefresh=true;
+        refreshes++;
         Redraw();
       }
       else if (model==vAdjustment->GetTopModel()) {
+        refreshes++;
         Redraw();
       }
       else if (model==hAdjustment->GetTopModel()) {
         fullRefresh=true;
-        Redraw();
-      }
-      else if (model==selection) {
+        refreshes++;
         Redraw();
       }
     }
