@@ -34,11 +34,11 @@ namespace Lum {
 
   ValueButton::ValueButton()
   : label(NULL),
-    value(new Text()),
     shortCut('\0'),scAssigned(false),
-    action(new Lum::Model::Action()),
+    action(new Model::Action()),
     triggering(false),
-    popup(NULL)
+    popup(NULL),
+    paintingValue(false)
   {
     SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
 
@@ -48,9 +48,6 @@ namespace Lum {
 
     SetRedrawOnMouseActive(true);
 
-    value->SetParent(this);
-    value->SetFlex(true,false);
-
     Observe(action);
   }
 
@@ -58,36 +55,17 @@ namespace Lum {
   {
     delete popup;
     delete label;
-    delete value;
   }
 
-  void ValueButton::ShowModelValue()
+  void ValueButton::SetPainter(ValuePainter* painter)
   {
-    Lum::Base::ModelRef model=GetModel();
-
-    value->SetFont(OS::display->GetFont(70));
-    value->SetStyle(Text::smart);
-    //value->SetAlignment(Text::centered);
-    value->SetText(GetModelValue());
+    this->painter=painter;
   }
 
-  std::wstring ValueButton::GetModelValue() const
+  void ValueButton::PreparePainter(ValuePainter* painter)
   {
-    if (GetModel()==NULL || GetModel()->IsNull()) {
-      return L"";
-    }
-    else if (dynamic_cast<Model::String*>(GetModel())!=NULL) {
-      return dynamic_cast<Model::String*>(GetModel())->Get();
-    }
-    else if (dynamic_cast<Model::Calendar*>(GetModel())!=NULL) {
-      return dynamic_cast<Model::Calendar*>(GetModel())->Get().GetLocaleDate();
-    }
-    else if (dynamic_cast<Model::Time*>(GetModel())!=NULL) {
-      return dynamic_cast<Model::Time*>(GetModel())->Get().GetLocaleTime();
-    }
-    else {
-      return L"";
-    }
+    painter->SetParent(this);
+    painter->SetModel(GetModel());
   }
 
   bool ValueButton::HasBaseline() const
@@ -102,6 +80,26 @@ namespace Lum {
     return GetTopBorder()+label->GetBaseline();
   }
 
+  OS::Font* ValueButton::GetFont(const void* child) const
+  {
+    if (child!=NULL && child==painter && paintingValue) {
+      return OS::display->GetFont(70);
+    }
+    else {
+      return Control::GetFont(child);
+    }
+  }
+
+  bool ValueButton::GetFontStyleSmart(const void* child) const
+  {
+    if (child!=NULL && child==painter && paintingValue) {
+      return true;
+    }
+    else {
+      return Control::GetFontStyleSmart(child);
+    }
+  }
+
   bool ValueButton::HasImplicitLabel() const
   {
     return true;
@@ -130,7 +128,6 @@ namespace Lum {
     text=new Text();
     text->SetParent(this);
     text->SetFlex(true,false);
-    //text->SetAlignment(Text::centered);
     text->SetText(string);
 
     delete label;
@@ -151,6 +148,9 @@ namespace Lum {
 
   void ValueButton::CalcSize()
   {
+    assert(label!=NULL);
+    assert(painter.Valid());
+
     SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
     if (!OS::display->GetTheme()->FocusOnEditableControlsOnly()) {
       RequestFocus();
@@ -170,26 +170,23 @@ namespace Lum {
       Now we let the image calculate its bounds and simply add its size
       to the size of the button.
     */
-    if (label!=NULL) {
-      label->CalcSize();
-      minWidth=std::max(minWidth,label->GetOMinWidth());
-      width=std::max(width,label->GetOWidth());
-      minHeight+=label->GetOMinHeight();
-      height+=label->GetOHeight();
-    }
+    label->CalcSize();
+    minWidth=std::max(minWidth,label->GetOMinWidth());
+    width=std::max(width,label->GetOWidth());
+    minHeight+=label->GetOMinHeight();
+    height+=label->GetOHeight();
 
-    if (value!=NULL) {
-      value->CalcSize();
-      minWidth=std::max(minWidth,value->GetOMinWidth());
-      width=std::max(width,value->GetOWidth());
-      minHeight+=value->GetOMinHeight();
-      height+=value->GetOHeight();
-    }
+    PreparePainter(painter);
+    paintingValue=true;
+    minWidth=std::max(minWidth,painter->GetProposedWidth());
+    width=std::max(width,painter->GetProposedWidth());
+    minHeight+=painter->GetProposedHeight();
+    height+=painter->GetProposedHeight();
+    paintingValue=false;
 
     maxWidth=30000;
     maxHeight=30000;
 
-    /* We *must* call CalcSize of our superclass! */
     Control::CalcSize();
   }
 
@@ -297,13 +294,6 @@ namespace Lum {
       label->Layout();
     }
 
-    if (value!=NULL) {
-      value->Resize(width,value->GetOHeight());
-      value->Move(x+(width-value->GetOWidth()) / 2,
-                  y+height-value->GetOHeight());
-      value->Layout();
-    }
-
     Control::Layout();
   }
 
@@ -324,16 +314,15 @@ namespace Lum {
     draw->focused=HasFocus();
   }
 
-  void ValueButton::Draw(int x, int y, size_t w, size_t h)
+  void ValueButton::Draw(OS::DrawInfo* draw,
+                         int x, int y, size_t w, size_t h)
   {
-    Control::Draw(x,y,w,h); /* We must call Draw of our superclass */
+    Control::Draw(draw,x,y,w,h); /* We must call Draw of our superclass */
 
     if (!OIntersect(x,y,w,h)) {
       return;
     }
 
-    OS::DrawInfo *draw=GetDrawInfo();
-
     /*
       Set the correct draw mode before calling the baseclass,
       since the baseclass draw the object frame.
@@ -352,12 +341,15 @@ namespace Lum {
 
     draw->focused=HasFocus();
 
-    if (label!=NULL) {
-      label->Draw(x,y,w,h);
-    }
+    label->Draw(draw,x,y,w,h);
 
-    if (value!=NULL) {
-      value->Draw(x,y,w,h);
+    if (!GetModel()->IsNull()) {
+      PreparePainter(painter);
+      paintingValue=true;
+      painter->Draw(draw,
+                    this->x,this->y+label->GetOHeight(),
+                    this->width,this->height-label->GetOHeight());
+      paintingValue=false;
     }
 
     draw->activated=false;
@@ -382,13 +374,7 @@ namespace Lum {
   void ValueButton::Hide()
   {
     if (visible) {
-      if (label!=NULL) {
-        label->Hide();
-      }
-
-      if (value!=NULL) {
-        value->Hide();
-      }
+      label->Hide();
 
       Control::Hide();
     }
@@ -420,7 +406,7 @@ namespace Lum {
       }
     }
     else if (model==GetModel()) {
-      ShowModelValue();
+      Redraw();
     }
 
     Control::Resync(model,msg);
@@ -428,98 +414,73 @@ namespace Lum {
 
   DateValueButton::DateValueButton()
   {
+    SetPainter(new DatePainter());
     SetPopup(new Dlg::DateValuePopup());
   }
 
-  std::wstring TimeValueButton::GetModelValue() const
-  {
-    if (GetModel()==NULL || GetModel()->IsNull()) {
-      return L"";
-    }
-    else if (dynamic_cast<Model::Time*>(GetModel())!=NULL) {
-      return dynamic_cast<Model::Time*>(GetModel())->Get().GetLocaleTime(format);
-    }
-    else {
-      return L"";
-    }
-  }
-
   TimeValueButton::TimeValueButton(Base::TimeFormat format)
    : format(format)
   {
+    SetPainter(new TimePainter(format));
     SetPopup(new Dlg::TimeValuePopup(format));
   }
 
-  TableStringValueButton::TableStringValueButton(Lum::Model::Table* table,
-                                                 Lum::Model::Header* header)
+  TableStringValueButton::TableStringValueButton(Model::Table* table,
+                                                 Model::Header* header)
   : tableModel(table),
-     headerModel(header)
+    headerModel(header)
   {
     Observe(table);
 
+    SetPainter(new StringPainter());
     SetPopup(new Dlg::TableStringValuePopup(table,header));
   }
 
-  void TableStringValueButton::Resync(Base::Model* model, const Base::ResyncMsg& msg)
-  {
-    if (model==tableModel) {
-      ShowModelValue();
-      Redraw();
-    }
-
-    ValueButton::Resync(model,msg);
-  }
-
-  TableIndexValueButton::TableIndexValueButton(Lum::Model::Table* table,
-                                               Lum::Model::Header* header)
+  TableIndexValueButton::TableIndexValueButton(Model::Table* table,
+                                               TableCellPainter* painter,
+                                               Model::Header* header)
   : tableModel(table),
-     headerModel(header)
+    headerModel(header)
   {
     Observe(table);
 
-    SetPopup(new Dlg::TableIndexValuePopup(table,header));
+    if (painter==NULL) {
+      painter=TableCellPainter::GetDefaultTableCellPainter(table);
+    }
+
+    SetPainter(painter);
+    SetPopup(new Dlg::TableIndexValuePopup(table,painter,header));
   }
 
-  std::wstring TableIndexValueButton::GetModelValue() const
+  void TableIndexValueButton::PreparePainter(ValuePainter* painter)
   {
-    if (GetModel()==NULL ||
-        GetModel()->IsNull() ||
-        !tableModel.Valid() ||
-        tableModel->GetRows()==0) {
-      return L"";
-    }
-    else if (dynamic_cast<Model::SizeT*>(GetModel())!=NULL) {
-      return tableModel->GetString(1,dynamic_cast<Model::SizeT*>(GetModel())->Get());
-    }
-    else {
-      return L"";
+    ValueButton::PreparePainter(painter);
+
+    Model::SizeT     *model=dynamic_cast<Model::SizeT*>(GetModel());
+    TableCellPainter *tablePainter=dynamic_cast<TableCellPainter*>(painter);
+
+    tablePainter->SetModel(tableModel);
+
+    if (model!=NULL && !model->IsNull()) {
+      tablePainter->SetCell(1,model->Get());
     }
   }
 
-  void TableIndexValueButton::Resync(Base::Model* model, const Base::ResyncMsg& msg)
-  {
-    if (model==tableModel) {
-      ShowModelValue();
-      Redraw();
-    }
-
-    ValueButton::Resync(model,msg);
-  }
-  
   TableIndexValueButton* TableIndexValueButton::Create(const std::wstring& label,
                                                        Base::Model* model,
-                                                       Lum::Model::Table* table,
+                                                       Model::Table* table,
+                                                       TableCellPainter* painter,
                                                        bool horizontalFlex,
                                                        bool verticalFlex)
   {
     TableIndexValueButton *control;
-    
-    control=new TableIndexValueButton(table);
-    
+
+    control=new TableIndexValueButton(table,painter);
+
     control->SetLabel(label);
     control->SetModel(model);
     control->SetFlex(horizontalFlex,verticalFlex);
-    
+
     return control;
-  }                                                                          
+  }
 }
