@@ -24,7 +24,7 @@
 
 #include <Lum/Base/Util.h>
 
-#include <Lum/OS/Behaviour.h>
+#include <Lum/Manager/Behaviour.h>
 
 #include <Lum/PosMarker.h>
 #include <Lum/Scroller.h>
@@ -36,12 +36,9 @@ namespace Lum {
     hScroller(NULL),vScroller(NULL),
     kineticScroller(new KineticScroller()),
     hVisible(false),vVisible(false),
-    kineticScrolling(true),
-    refreshAction(new Model::Action())
+    kineticScrolling(true)
   {
     SetBackground(OS::display->GetFill(OS::Display::scrolledBackgroundFillIndex));
-
-    Observe(refreshAction);
   }
 
   View::~View()
@@ -85,8 +82,8 @@ namespace Lum {
     assert(object!=NULL);
 
     if (this->object!=NULL) {
-      Forget(object->GetHAdjustment());
-      Forget(object->GetVAdjustment());
+      Forget(object->GetHAdjustment()->GetTotalModel());
+      Forget(object->GetVAdjustment()->GetTotalModel());
       delete this->object;
     }
 
@@ -95,8 +92,8 @@ namespace Lum {
     object->SetParent(this);
     object->SetFlex(true,true);
 
-    Observe(object->GetHAdjustment());
-    Observe(object->GetVAdjustment());
+    Observe(object->GetHAdjustment()->GetTotalModel());
+    Observe(object->GetVAdjustment()->GetTotalModel());
 
     kineticScroller->SetObjects(this,this->object);
   }
@@ -115,87 +112,121 @@ namespace Lum {
   {
     assert(object!=NULL);
 
+    object->SetScrollView(this);
+
+    if (hScroller==NULL) {
+      if (!Manager::Behaviour::Instance()->ScrollKnobActive() &&
+          !object->RequiresKnobs()) {
+        hScroller=new PosMarker(false);
+      }
+      else {
+        hScroller=new Scroller(false);
+      }
+
+      hScroller->SetParent(this);
+      hScroller->SetFlex(true,false);
+      hScroller->SetModel(object->GetHAdjustment());
+    }
+
+    if (vScroller==NULL) {
+      if (!Manager::Behaviour::Instance()->ScrollKnobActive() &&
+          !object->RequiresKnobs()) {
+        vScroller=new PosMarker(true);
+      }
+      else {
+        vScroller=new Scroller(true);
+      }
+
+      vScroller->SetParent(this);
+      vScroller->SetFlex(false,true);
+      vScroller->SetModel(object->GetVAdjustment());
+    }
+
+    hScroller->CalcSize();
+    vScroller->CalcSize();
     object->CalcSize();
 
-    if (!OS::display->GetBehaviour()->ScrollKnobActive() &&
-        !object->RequiresKnobs() &&
-        kineticScrolling) {
-      vScroller=new PosMarker(true);
-      hScroller=new PosMarker(false);
-    }
-    else {
-      vScroller=new Scroller(true);
-      hScroller=new Scroller(false);
-    }
-
-    vScroller->SetParent(this);
-    vScroller->SetFlex(false,true);
-    vScroller->SetModel(object->GetVAdjustment());
-
-    hScroller->SetParent(this);
-    hScroller->SetFlex(true,false);
-    hScroller->SetModel(object->GetHAdjustment());
-
     if (object->RequestedFocus()) {
       /* Delegate focusing to the real table */
       UnrequestFocus();
     }
 
+    // Initial dimension equal to table dimension
     minWidth=object->GetOMinWidth();
     width=object->GetOWidth();
     minHeight=object->GetOMinHeight();
     height=object->GetOHeight();
 
-    hScroller->CalcSize();
-
+    // Add Horizontal scroller
     minWidth=std::max(hScroller->GetOWidth(),minWidth);
     width=std::max(hScroller->GetOWidth(),width);
     minHeight+=hScroller->GetOHeight();
-    height+=hScroller->GetOHeight();
 
-    vScroller->CalcSize();
 
+    // Add Vertical scroller
     minWidth+=vScroller->GetOMinWidth();
     width+=vScroller->GetOWidth();
     minHeight=std::max(vScroller->GetOHeight(),minHeight);
     height=std::max(vScroller->GetOHeight(),height);
 
+    maxWidth=std::min(maxWidth,object->GetOMaxWidth()+vScroller->GetOMaxWidth());
+    maxHeight=std::min(maxHeight,object->GetOMaxHeight()+hScroller->GetOMaxHeight());
+
     Group::CalcSize();
   }
 
   void View::Layout()
   {
-    hScroller->ResizeWidth(GetOWidth());
-    hScroller->Layout();
-    vScroller->ResizeHeight(GetOHeight());
-    vScroller->Layout();
-
-    if (hVisible &&
-        (!object->GetHAdjustment()->IsValid() ||
-         object->GetHAdjustment()->IsCompletelyVisible())) {
-      hScroller->Hide();
-      hVisible=false;
-    }
-    else if (!hVisible &&
-             object->GetHAdjustment()->IsValid() &&
-             !object->GetHAdjustment()->IsCompletelyVisible()) {
-      hVisible=true;
-    }
-
-    if (vVisible &&
-        (!object->GetVAdjustment()->IsValid() ||
-         object->GetVAdjustment()->IsCompletelyVisible())) {
-      vScroller->Hide();
-      vVisible=false;
-    }
-    else if (!vVisible &&
-             object->GetVAdjustment()->IsValid() &&
-             !object->GetVAdjustment()->IsCompletelyVisible()) {
-      vVisible=true;
-    }
-
     size_t tWidth,tHeight;
 
+    hScroller->ResizeWidth(GetOWidth());
+    vScroller->ResizeHeight(GetOHeight());
+
+    tWidth=width;
+    tHeight=height;
+    if (object->GetBackground()!=NULL) {
+      tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+      tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+    }
+    object->GetODimension(tWidth,tHeight);
+    if (object->GetBackground()!=NULL) {
+      tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+      tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+    }
+
+    hVisible=width<tWidth;
+    vVisible=height<tHeight;
+
+    // Showing one scrollbar reduces the visible table bar, so the other scrollbar
+    // might be shown, too
+
+    if (hVisible && !vVisible) {
+      tHeight=height-hScroller->GetOHeight();
+      if (object->GetBackground()!=NULL) {
+        tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      object->GetODimension(tWidth,tHeight);
+      if (object->GetBackground()!=NULL) {
+        tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      vVisible=height-hScroller->GetOHeight()<tHeight;
+    }
+    else if (!hVisible && vVisible) {
+      tWidth=width-vScroller->GetOWidth();
+      if (object->GetBackground()!=NULL) {
+        tWidth-=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight-=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      object->GetODimension(tWidth,tHeight);
+      if (object->GetBackground()!=NULL) {
+        tWidth+=object->GetBackground()->leftBorder+object->GetBackground()->rightBorder;
+        tHeight+=object->GetBackground()->topBorder+object->GetBackground()->bottomBorder;
+      }
+      hVisible=width-vScroller->GetOWidth()<tWidth;
+    }
+
     tWidth=width;
     tHeight=height;
 
@@ -209,18 +240,21 @@ namespace Lum {
       tWidth-=vScroller->GetOWidth();
     }
 
-    // Resize and position objects
-
-    object->MoveResize(x,y,tWidth,tHeight);
-    object->Layout();
-
     if (hVisible) {
       hScroller->MoveResize(x,y+tHeight,tWidth,hScroller->GetOHeight());
     }
+    else {
+      hScroller->Hide();
+    }
 
     if (vVisible) {
       vScroller->MoveResize(x+tWidth,y,vScroller->GetOWidth(),tHeight);
     }
+    else {
+      vScroller->Hide();
+    }
+
+    object->MoveResize(x,y,tWidth,tHeight);
 
     Group::Layout();
   }
@@ -251,32 +285,9 @@ namespace Lum {
       return;
     }
 
-    if (model==object->GetHAdjustment() ||
-        model==object->GetVAdjustment()) {
-      if (visible) {
-        OS::display->QueueActionForEventLoop(refreshAction);
-      }
-      else {
-        SetRelayout();
-      }
-    }
-    else if (model==refreshAction &&
-             refreshAction->IsFinished()) {
-      if ((hVisible &&
-           (!object->GetHAdjustment()->IsValid() ||
-            object->GetHAdjustment()->IsCompletelyVisible())) ||
-          (!hVisible &&
-           object->GetHAdjustment()->IsValid() &&
-           !object->GetHAdjustment()->IsCompletelyVisible()) ||
-          (vVisible &&
-           (!object->GetVAdjustment()->IsValid() ||
-            object->GetVAdjustment()->IsCompletelyVisible())) ||
-          (!vVisible &&
-           object->GetVAdjustment()->IsValid() &&
-           !object->GetVAdjustment()->IsCompletelyVisible())) {
-        SetRelayout();
-        Redraw();
-      }
+    if (model==object->GetHAdjustment()->GetTotalModel() ||
+        model==object->GetVAdjustment()->GetTotalModel()) {
+      SetRelayout();
     }
 
     if (!object->RequiresKnobs() &&
